from deficrawler.lending import Lending


def test_redeem_aave_2_eth():
    aave = Lending(protocol="Aave", chain="Ethereum", version=2)
    redeem = aave.get_data_from_date_range(
        '30/08/2021 00:00:01', '30/08/2021 18:01:00', "redeem")

    assert(redeem[0]['tx_id'] != "")
    assert(redeem[0]['protocol'] == "Aave")
    assert(redeem[0]['chain'] == "Ethereum")
    assert(redeem[0]['version'] == 2)
    assert(redeem[0]['user'] != "")
    assert(redeem[0]['token'] != "")
    assert(redeem[0]['amount'] > 0)
    assert(redeem[0]['timestamp'] > 0)


def test_redeem_aave_2_eth_user():
    aave = Lending(protocol="Aave", chain="Ethereum", version=2)
    redeems = aave.get_data_from_date_range(
        '30/08/2021 00:00:01', '31/08/2021 18:01:00', "redeem", "0x27239549dd40e1d60f5b80b0c4196923745b1fd2")

    for redeem in redeems:
        assert(redeem['user'] ==
               "0x27239549dd40e1d60f5b80b0c4196923745b1fd2")


def test_redeem_aave_2_polygon():
    aave = Lending(protocol="Aave", chain="Polygon", version=2)
    redeem = aave.get_data_from_date_range(
        '30/08/2021 00:00:01', '30/08/2021 02:01:00', "redeem")
    assert(redeem[0]['tx_id'] != "")
    assert(redeem[0]['protocol'] == "Aave")
    assert(redeem[0]['chain'] == "Polygon")
    assert(redeem[0]['version'] == 2)
    assert(redeem[0]['user'] != "")
    assert(redeem[0]['token'] != "")
    assert(redeem[0]['amount'] > 0)
    assert(redeem[0]['timestamp'] > 0)


def test_redeem_aave_2_avalanche():
    aave = Lending(protocol="Aave", chain="Avalanche", version=2)
    redeem = aave.get_data_from_date_range(
        '17/10/2021 00:00:01', '17/10/2021 09:01:00', "redeem")
    assert(redeem[0]['tx_id'] != "")
    assert(redeem[0]['protocol'] == "Aave")
    assert(redeem[0]['chain'] == "Avalanche")
    assert(redeem[0]['version'] == 2)
    assert(redeem[0]['user'] != "")
    assert(redeem[0]['token'] != "")
    assert(redeem[0]['amount'] > 0)
    assert(redeem[0]['timestamp'] > 0)


def test_redeem_aave_2_polygon_user():
    aave = Lending(protocol="Aave", chain="Polygon", version=2)
    redeems = aave.get_data_from_date_range(
        '30/08/2021 00:00:01', '31/08/2021 02:01:00', "redeem", "0x3fcd5de6a9fc8a99995c406c77dda3ed7e406f81")

    for redeem in redeems:
        assert(redeem['user'] ==
               "0x3fcd5de6a9fc8a99995c406c77dda3ed7e406f81")


def test_redeem_compound_2_eth():
    compound = Lending(protocol="Compound", chain="Ethereum", version=2)
    redeem = compound.get_data_from_date_range(
        '28/07/2021 00:00:01', '30/07/2021 18:01:00', "redeem")

    assert(redeem[0]['tx_id'] != "")
    assert(redeem[0]['protocol'] == "Compound")
    assert(redeem[0]['chain'] == "Ethereum")
    assert(redeem[0]['version'] == 2)
    assert(redeem[0]['user'] != "")
    assert(redeem[0]['token'] != "")
    assert(float(redeem[0]['amount']) > 0)
    assert(redeem[0]['timestamp'] > 0)


def test_redeem_compound_2_eth_user():
    compound = Lending(protocol="Compound", chain="Ethereum", version=2)
    redeems = compound.get_data_from_date_range(
        '28/07/2021 00:00:01', '3/07/2021 18:01:00', "redeem", "0x4ddc2d193948926d02f9b1fe9e1daa0718270ed5")

    for redeem in redeems:
        assert(redeem['user'] ==
               "0x4ddc2d193948926d02f9b1fe9e1daa0718270ed5")


def test_redeem_cream_2_eth():
    cream = Lending(protocol="Cream", chain="Ethereum", version=2)
    redeem = cream.get_data_from_date_range(
        '28/07/2021 00:00:01', '30/07/2021 18:01:00', "redeem")

    assert(redeem[0]['tx_id'] != "")
    assert(redeem[0]['protocol'] == "Cream")
    assert(redeem[0]['chain'] == "Ethereum")
    assert(redeem[0]['version'] == 2)
    assert(redeem[0]['user'] != "")
    assert(redeem[0]['token'] != "")
    assert(float(redeem[0]['amount']) > 0)
    assert(redeem[0]['timestamp'] > 0)


def test_redeem_cream_2_polygon():
    cream = Lending(protocol="Cream", chain="Polygon", version=2)
    redeem = cream.get_data_from_date_range(
        '25/09/2021 00:00:01', '26/09/2021 18:01:00', "redeem")

    assert(redeem[0]['tx_id'] != "")
    assert(redeem[0]['protocol'] == "Cream")
    assert(redeem[0]['chain'] == "Polygon")
    assert(redeem[0]['version'] == 2)
    assert(redeem[0]['user'] != "")
    assert(redeem[0]['token'] != "")
    assert(float(redeem[0]['amount']) > 0)
    assert(redeem[0]['timestamp'] > 0)


def test_redeem_cream_2_arbitrum():
    cream = Lending(protocol="Cream", chain="Arbitrum", version=2)
    redeem = cream.get_data_from_date_range(
        '25/09/2021 00:00:01', '26/09/2021 18:01:00', "redeem")

    assert(redeem[0]['tx_id'] != "")
    assert(redeem[0]['protocol'] == "Cream")
    assert(redeem[0]['chain'] == "Arbitrum")
    assert(redeem[0]['version'] == 2)
    assert(redeem[0]['user'] != "")
    assert(redeem[0]['token'] != "")
    assert(float(redeem[0]['amount']) > 0)
    assert(redeem[0]['timestamp'] > 0)


def test_redeem_cream_2_fantom():
    cream = Lending(protocol="Cream", chain="fantom", version=2)
    redeem = cream.get_data_from_date_range(
        '18/10/2021 00:00:01', '19/10/2021 00:00:00', "redeem")

    assert(redeem[0]['tx_id'] != "")
    assert(redeem[0]['protocol'] == "Cream")
    assert(redeem[0]['chain'] == "fantom")
    assert(redeem[0]['version'] == 2)
    assert(redeem[0]['user'] != "")
    assert(redeem[0]['token'] != "")
    assert(float(redeem[0]['amount']) > 0)
    assert(redeem[0]['timestamp'] > 0)


def test_redeem_cream_2_avalanche():
    cream = Lending(protocol="Cream", chain="avalanche", version=2)
    redeem = cream.get_data_from_date_range(
        '18/10/2021 00:00:01', '19/10/2021 00:00:00', "redeem")

    assert(redeem[0]['tx_id'] != "")
    assert(redeem[0]['protocol'] == "Cream")
    assert(redeem[0]['chain'] == "avalanche")
    assert(redeem[0]['version'] == 2)
    assert(redeem[0]['user'] != "")
    assert(redeem[0]['token'] != "")
    assert(float(redeem[0]['amount']) > 0)
    assert(redeem[0]['timestamp'] > 0)


def test_redeem_cream_2_eth_user():
    cream = Lending(protocol="Cream", chain="Ethereum", version=2)
    redeems = cream.get_data_from_date_range(
        '28/07/2021 00:00:01', '30/07/2021 18:01:00', "redeem", "0x85759961b116f1d36fd697855c57a6ae40793d9b")

    for redeem in redeems:
        assert(redeem['user'] ==
               "0x85759961b116f1d36fd697855c57a6ae40793d9b")


def test_redeem_cream_2_bsc():
    cream = Lending(protocol="Cream", chain="Bsc", version=2)
    redeem = cream.get_data_from_date_range(
        '28/07/2021 00:00:01', '30/07/2021 18:01:00', "redeem")

    assert(redeem[0]['tx_id'] != "")
    assert(redeem[0]['protocol'] == "Cream")
    assert(redeem[0]['chain'] == "Bsc")
    assert(redeem[0]['version'] == 2)
    assert(redeem[0]['user'] != "")
    assert(redeem[0]['token'] != "")
    assert(float(redeem[0]['amount']) > 0)
    assert(redeem[0]['timestamp'] > 0)


def test_redeem_cream_2_bsc_user():
    cream = Lending(protocol="Cream", chain="Bsc", version=2)
    redeems = cream.get_data_from_date_range(
        '28/07/2021 00:00:01', '30/07/2021 18:01:00', "redeem", "0x1ffe17b99b439be0afc831239ddecda2a790ff3a")

    for redeem in redeems:
        assert(redeem['user'] ==
               "0x1ffe17b99b439be0afc831239ddecda2a790ff3a")


def test_redeem_kashi_1_eth():
    kashi = Lending(protocol="Kashi", chain="Ethereum", version=1)
    redeem = kashi.get_data_from_date_range(
        '25/09/2021 00:00:01', '30/09/2021 18:01:00', "redeem")

    assert(redeem[0]['tx_id'] != "")
    assert(redeem[0]['protocol'] == "Kashi")
    assert(redeem[0]['chain'] == "Ethereum")
    assert(redeem[0]['version'] == 1)
    assert(redeem[0]['user'] != "")
    assert(redeem[0]['token'] != "")
    assert(float(redeem[0]['amount']) > 0)
    assert(redeem[0]['timestamp'] > 0)


def test_redeem_kashi_1_polygon():
    kashi = Lending(protocol="Kashi", chain="Polygon", version=1)
    redeem = kashi.get_data_from_date_range(
        '25/09/2021 00:00:01', '30/09/2021 18:01:00', "redeem")

    assert(redeem[0]['tx_id'] != "")
    assert(redeem[0]['protocol'] == "Kashi")
    assert(redeem[0]['chain'] == "Polygon")
    assert(redeem[0]['version'] == 1)
    assert(redeem[0]['user'] != "")
    assert(redeem[0]['token'] != "")
    assert(float(redeem[0]['amount']) > 0)
    assert(redeem[0]['timestamp'] > 0)



from datetime import time


def get_attributes(entity, map_file):
    """
    Returns the list of attributes for the given map file and entity.
    The list is created joining the attributes of the entity and the 
    extra fields (if exists) to create the query
    """
    list_attr = ''
    attributes = map_file['entities'][entity]['query']['extra_fields']
    attributes.update(map_file['entities'][entity]['attributes'])
    for attribute, value in attributes.items():
        list_attr += format_attribute(value) + " "
    return list_attr


def get_filters(filter_dict):
    """
    Returns the filters list in string mode for a given dictionary.
    Gets all the filters that the dictionary contains and create a list 
    of key:value elements
    """
    filters = ''

    if type(filter_dict) is dict:
        for filter_name, filter_value in filter_dict.items():
            filters += filter_name + ":\"" + filter_value + "\"\n"

    return filters


def format_element(list_elements):
    """
    Formats the element to query the subgraph. 
    If only one element is given returns the element, in other case
    format the element with the format element{subelement{subsubelement}}
    """
    if(len(list_elements) == 1):
        return list_elements[0]

    path = list_elements[0] + '{'
    for index, field in enumerate(list_elements):
        if index > 0 and index % 2 != 0:
            path += field
        if index > 0 and index % 2 == 0:
            path += '{'+field+'}'

    path += '}'
    return path


def format_attribute(list_fields):
    """
    Format the attribute to base on the information of the json file.
    For each field calls the format element, can be a single list wich 
    returns a single element or a list of lists that will concatenate the 
    strings for each list
    """
    str_list_attr = ""

    if(isinstance(list_fields[0], list)):
        for list_elements in list_fields:
            str_list_attr += format_element(list_elements) + " "
    else:
        str_list_attr += format_element(list_fields)
    return str_list_attr


def filter_method(block, order_by_filter, lte, timestamp):
    """
    Returns the block filter or the where filter depending of the block passed
    """
    block_ret = ''
    order_by_filter_ret = ''
    lte_ret = ''
    timestamp_ret = ''

    if block:
        block_ret = "block:{{ number: {number} }}".format(number=block)
    else:
        order_by_filter_ret = order_by_filter
        lte_ret = lte
        timestamp_ret = timestamp

    return {
        'block': block_ret,
        'order_by_filter': order_by_filter_ret,
        'lte': lte_ret,
        'timestamp': timestamp_ret
    }



from deficrawler.lending import Lending


def test_flashloans_aave_2_eth():
    aave = Lending(protocol="Aave", chain="Ethereum", version=2)
    flash_loans = aave.get_data_from_date_range(
        '30/08/2021 00:00:01', '30/08/2021 18:01:00', "flashloans")

    assert(flash_loans[0]['tx_id'] != "")
    assert(flash_loans[0]['protocol'] == "Aave")
    assert(flash_loans[0]['chain'] == "Ethereum")
    assert(flash_loans[0]['version'] == 2)
    assert(flash_loans[0]['user'] != "")
    assert(flash_loans[0]['token'] != "")
    assert(flash_loans[0]['amount'] > 0)
    assert(flash_loans[0]['timestamp'] > 0)


def test_flashloans_aave_2_avalanche():
    aave = Lending(protocol="Aave", chain="Avalanche", version=2)
    flash_loans = aave.get_data_from_date_range(
        '10/10/2021 00:00:01', '18/10/2021 18:01:00', "flashloans")

    assert(flash_loans[0]['tx_id'] != "")
    assert(flash_loans[0]['protocol'] == "Aave")
    assert(flash_loans[0]['chain'] == "Avalanche")
    assert(flash_loans[0]['version'] == 2)
    assert(flash_loans[0]['user'] != "")
    assert(flash_loans[0]['token'] != "")
    assert(flash_loans[0]['amount'] > 0)
    assert(flash_loans[0]['timestamp'] > 0)


def test_flashloans_aave_2_eth_user():
    aave = Lending(protocol="Aave", chain="Ethereum", version=2)
    flash_loans = aave.get_data_from_date_range(
        '30/08/2021 00:00:01', '30/08/2021 18:01:00', "flashloans", "0x4f2769e87c7d96ed9ca72084845ee05e7de5dda2")

    for flash_loan in flash_loans:
        assert(flash_loan['user'] ==
               "0x4f2769e87c7d96ed9ca72084845ee05e7de5dda2")


def test_flashloans_aave_2_polygon():
    aave = Lending(protocol="Aave", chain="Polygon", version=2)
    flash_loans = aave.get_data_from_date_range(
        '30/08/2021 00:00:01', '30/08/2021 18:01:00', "flashloans")
    assert(flash_loans[0]['tx_id'] != "")
    assert(flash_loans[0]['protocol'] == "Aave")
    assert(flash_loans[0]['chain'] == "Polygon")
    assert(flash_loans[0]['version'] == 2)
    assert(flash_loans[0]['user'] != "")
    assert(flash_loans[0]['token'] != "")
    assert(flash_loans[0]['amount'] > 0)
    assert(flash_loans[0]['timestamp'] > 0)


def test_flashloans_aave_2_polygon_user():
    aave = Lending(protocol="Aave", chain="Polygon", version=2)
    flash_loans = aave.get_data_from_date_range(
        '30/08/2021 00:00:01', '30/08/2021 18:01:00', "flashloans", "0xabcd3c5e8aed3b8d8096f0f33c7aa1cb5d555dfb")

    for flash_loan in flash_loans:
        assert(flash_loan['user'] ==
               "0xabcd3c5e8aed3b8d8096f0f33c7aa1cb5d555dfb")



class Querys:
    QUERY_FROM_TIMESTAMP = """{{
                {entity_name}(
                            first: 1000
                            orderBy: {order_by}
                            orderDirection: asc
                            where: {{
                                {order_by}_gt: {from_timestamp}
                                {order_by}_lte: {to_timestamp}
                                {aditional_filters}
                            }}
                        ){{
                        {attributes}
                    }}
                    }}
                """

    QUERY_ALL_ELEMENTS = """ {{
            {entity_name}(
                    first: 1000
                    orderBy: {order_by}
                    orderDirection: asc
                    where: {{
                        {order_by}_gt:"{filter_value}"
                    }}
                ){{
                    {attributes}
                }}
            }}
        """

    QUERY_ELEMENT_FILTER = """ {{
            {entity_name}(
                    first: 1000
                    where: {{
                        {filters}
                    }}
                ){{
                    {attributes}
                }}
            }}
        """

    QUERY_FIRST_ELEMENT = """ {{
            {entity_name}(
                    orderBy: {order_by}
                    orderDirection: desc
                    first: 1
                    {block}
                    where: {{
                        {order_by_filter}{lte} {timestamp}
                        {aditional_filters}
                    }}
                ){{
                    {attributes}
                }}
            }}
        """



from deficrawler.lending import Lending


def test_deposit_aave_2_eth():
    aave = Lending(protocol="Aave", chain="Ethereum", version=2)
    deposits = aave.get_data_from_date_range(
        '10/05/2021 00:00:01', '11/05/2021 00:01:00', "deposit")
    assert(deposits[0]['tx_id'] != "")
    assert(deposits[0]['protocol'] == "Aave")
    assert(deposits[0]['chain'] == "Ethereum")
    assert(deposits[0]['version'] == 2)
    assert(deposits[0]['user'] != "")
    assert(deposits[0]['token'] != "")
    assert(deposits[0]['amount'] > 0)
    assert(deposits[0]['timestamp'] > 0)


def test_deposit_aave_2_eth_user():
    aave = Lending(protocol="Aave", chain="Ethereum", version=2)
    deposits = aave.get_data_from_date_range(
        '10/05/2021 00:00:01', '14/05/2021 00:01:00', "deposit", "0x63a3f444e97d14e671e7ee323c4234c8095e3516")

    for deposit in deposits:
        assert(deposit['user'] == "0x63a3f444e97d14e671e7ee323c4234c8095e3516")


def test_deposit_aave_2_polygon():
    aave = Lending(protocol="Aave", chain="Polygon", version=2)
    deposits = aave.get_data_from_date_range(
        '11/05/2021 00:00:01', '11/05/2021 00:01:10', "deposit")

    assert(deposits[0]['tx_id'] != "")
    assert(deposits[0]['protocol'] == "Aave")
    assert(deposits[0]['chain'] == "Polygon")
    assert(deposits[0]['version'] == 2)
    assert(deposits[0]['user'] != "")
    assert(deposits[0]['token'] != "")
    assert(deposits[0]['amount'] > 0)
    assert(deposits[0]['timestamp'] > 0)


def test_deposit_aave_2_avalanche():
    aave = Lending(protocol="Aave", chain="Avalanche", version=2)
    deposits = aave.get_data_from_date_range(
        '16/10/2021 00:00:01', '16/10/2021 09:01:10', "deposit")

    assert(deposits[0]['tx_id'] != "")
    assert(deposits[0]['protocol'] == "Aave")
    assert(deposits[0]['chain'] == "Avalanche")
    assert(deposits[0]['version'] == 2)
    assert(deposits[0]['user'] != "")
    assert(deposits[0]['token'] != "")
    assert(deposits[0]['amount'] > 0)
    assert(deposits[0]['timestamp'] > 0)


def test_deposit_aave_2_polygon_user():
    aave = Lending(protocol="Aave", chain="Polygon", version=2)
    deposits = aave.get_data_from_date_range(
        '10/05/2021 00:00:01', '12/05/2021 00:01:00', "deposit", "0xbeadf48d62acc944a06eeae0a9054a90e5a7dc97")

    for deposit in deposits:
        assert(deposit['user'] == "0xbeadf48d62acc944a06eeae0a9054a90e5a7dc97")


def test_deposit_compound_2_eth():
    compound = Lending(protocol="Compound", chain="Ethereum", version=2)
    deposits = compound.get_data_from_date_range(
        '11/05/2021 00:00:01', '11/05/2021 01:01:10', "deposit")

    assert(deposits[0]['tx_id'] != "")
    assert(deposits[0]['protocol'] == "Compound")
    assert(deposits[0]['chain'] == "Ethereum")
    assert(deposits[0]['version'] == 2)
    assert(deposits[0]['user'] != "")
    assert(deposits[0]['token'] != "")
    assert(float(deposits[0]['amount']) > 0)
    assert(deposits[0]['timestamp'] > 0)


def test_deposit_compound_2_eth_user():
    compound = Lending(protocol="Compound", chain="Ethereum", version=2)
    deposits = compound.get_data_from_date_range(
        '11/05/2021 00:00:01', '14/05/2021 00:01:10', "deposit", "0x862e840c2f7edd2c709d8de0df71c03210ed5267")

    for deposit in deposits:
        assert(deposit['user'] == "0x862e840c2f7edd2c709d8de0df71c03210ed5267")


def test_deposit_cream_2_eth():
    cream = Lending(protocol="Cream", chain="Ethereum", version=2)
    deposits = cream.get_data_from_date_range(
        '11/05/2021 00:00:01', '12/05/2021 11:54:10', "deposit")

    assert(deposits[0]['tx_id'] != "")
    assert(deposits[0]['protocol'] == "Cream")
    assert(deposits[0]['chain'] == "Ethereum")
    assert(deposits[0]['version'] == 2)
    assert(deposits[0]['user'] != "")
    assert(deposits[0]['token'] != "")
    assert(float(deposits[0]['amount']) > 0)
    assert(deposits[0]['timestamp'] > 0)

def test_deposit_cream_2_polygon():
    cream = Lending(protocol="Cream", chain="Polygon", version=2)
    deposits = cream.get_data_from_date_range(
        '25/09/2021 00:00:01', '26/09/2021 11:54:10', "deposit")

    assert(deposits[0]['tx_id'] != "")
    assert(deposits[0]['protocol'] == "Cream")
    assert(deposits[0]['chain'] == "Polygon")
    assert(deposits[0]['version'] == 2)
    assert(deposits[0]['user'] != "")
    assert(deposits[0]['token'] != "")
    assert(float(deposits[0]['amount']) > 0)
    assert(deposits[0]['timestamp'] > 0)



def test_deposit_cream_2_eth_user():
    cream = Lending(protocol="Cream", chain="Ethereum", version=2)
    deposits = cream.get_data_from_date_range(
        '11/05/2021 00:00:01', '14/05/2021 11:54:10', "deposit", "0xe6a6ee4196d361ec4f6d587c7ebe20c50667fb39")

    for deposit in deposits:
        assert(deposit['user'] == "0xe6a6ee4196d361ec4f6d587c7ebe20c50667fb39")


def test_deposit_cream_2_bsc():
    cream = Lending(protocol="Cream", chain="bsc", version=2)
    deposits = cream.get_data_from_date_range(
        '08/05/2021 00:00:01', '12/05/2021 11:54:10', "deposit")

    assert(deposits[0]['tx_id'] != "")
    assert(deposits[0]['protocol'] == "Cream")
    assert(deposits[0]['chain'] == "bsc")
    assert(deposits[0]['version'] == 2)
    assert(deposits[0]['user'] != "")
    assert(deposits[0]['token'] != "")
    assert(float(deposits[0]['amount']) > 0)
    assert(deposits[0]['timestamp'] > 0)


def test_deposit_cream_2_fantom():
    cream = Lending(protocol="Cream", chain="fantom", version=2)
    deposits = cream.get_data_from_date_range(
        '10/10/2021 00:00:01', '19/10/2021 00:00:00', "deposit")

    assert(deposits[0]['tx_id'] != "")
    assert(deposits[0]['protocol'] == "Cream")
    assert(deposits[0]['chain'] == "fantom")
    assert(deposits[0]['version'] == 2)
    assert(deposits[0]['user'] != "")
    assert(deposits[0]['token'] != "")
    assert(float(deposits[0]['amount']) > 0)
    assert(deposits[0]['timestamp'] > 0)


def test_deposit_cream_2_avalanche():
    cream = Lending(protocol="Cream", chain="avalanche", version=2)
    deposits = cream.get_data_from_date_range(
        '10/10/2021 00:00:01', '19/10/2021 00:00:00', "deposit")

    assert(deposits[0]['tx_id'] != "")
    assert(deposits[0]['protocol'] == "Cream")
    assert(deposits[0]['chain'] == "avalanche")
    assert(deposits[0]['version'] == 2)
    assert(deposits[0]['user'] != "")
    assert(deposits[0]['token'] != "")
    assert(float(deposits[0]['amount']) > 0)
    assert(deposits[0]['timestamp'] > 0)

def test_deposit_cream_2_bsc_user():
    cream = Lending(protocol="Cream", chain="bsc", version=2)
    deposits = cream.get_data_from_date_range(
        '08/05/2021 00:00:01', '14/05/2021 11:54:10', "deposit", "0xc6fb91e0ee3b1077c5465d54b3dda176a695199b")

    for deposit in deposits:
        assert(deposit['user'] == "0xc6fb91e0ee3b1077c5465d54b3dda176a695199b")


def test_deposit_kashi_1_eth():
    kashi = Lending(protocol="kashi", chain="Ethereum", version=1)
    deposits = kashi.get_data_from_date_range(
        '25/09/2021 00:00:01', '30/09/2021 11:54:10', "deposit")

    assert(deposits[0]['tx_id'] != "")
    assert(deposits[0]['protocol'] == "kashi")
    assert(deposits[0]['chain'] == "Ethereum")
    assert(deposits[0]['version'] == 1)
    assert(deposits[0]['user'] != "")
    assert(deposits[0]['token'] != "")
    assert(float(deposits[0]['amount']) > 0)
    assert(deposits[0]['timestamp'] > 0)


def test_deposit_kashi_1_polygon():
    kashi = Lending(protocol="kashi", chain="Polygon", version=1)
    deposits = kashi.get_data_from_date_range(
        '25/09/2021 00:00:01', '30/09/2021 11:54:10', "deposit")

    assert(deposits[0]['tx_id'] != "")
    assert(deposits[0]['protocol'] == "kashi")
    assert(deposits[0]['chain'] == "Polygon")
    assert(deposits[0]['version'] == 1)
    assert(deposits[0]['user'] != "")
    assert(deposits[0]['token'] != "")
    assert(float(deposits[0]['amount']) > 0)
    assert(deposits[0]['timestamp'] > 0)



from deficrawler.lending import Lending


def test_borrow_aave_2_eth():
    aave = Lending(protocol="Aave", chain="Ethereum", version=2)
    borrows = aave.get_data_from_date_range(
        '11/05/2021 00:00:01', '11/05/2021 01:01:00', "borrow")

    assert(borrows[0]['tx_id'] != "")
    assert(borrows[0]['protocol'] == "Aave")
    assert(borrows[0]['chain'] == "Ethereum")
    assert(borrows[0]['version'] == 2)
    assert(borrows[0]['user'] != "")
    assert(borrows[0]['token'] != "")
    assert(borrows[0]['amount'] > 0)
    assert(borrows[0]['timestamp'] > 0)


def test_borrow_aave_2_eth_user():
    aave = Lending(protocol="Aave", chain="Ethereum", version=2)
    borrows = aave.get_data_from_date_range(
        '11/05/2021 00:00:01', '12/05/2021 00:01:00', "borrow", "0x8b8cdf85e4378894310e8ec6878d0ca09f105429")

    for borrow in borrows:
        assert(borrow['user'] == "0x8b8cdf85e4378894310e8ec6878d0ca09f105429")


def test_borrow_aave_2_polygon():
    aave = Lending(protocol="Aave", chain="Polygon", version=2)
    borrows = aave.get_data_from_date_range(
        '11/05/2021 00:00:01', '11/05/2021 00:01:10', "borrow")

    assert(borrows[0]['tx_id'] != "")
    assert(borrows[0]['protocol'] == "Aave")
    assert(borrows[0]['chain'] == "Polygon")
    assert(borrows[0]['version'] == 2)
    assert(borrows[0]['user'] != "")
    assert(borrows[0]['token'] != "")
    assert(borrows[0]['amount'] > 0)
    assert(borrows[0]['timestamp'] > 0)


def test_borrow_aave_2_avalanche():
    aave = Lending(protocol="Aave", chain="avalanche", version=2)
    borrows = aave.get_data_from_date_range(
        '16/10/2021 00:00:01', '18/10/2021 00:01:10', "borrow")

    assert(borrows[0]['tx_id'] != "")
    assert(borrows[0]['protocol'] == "Aave")
    assert(borrows[0]['chain'] == "avalanche")
    assert(borrows[0]['version'] == 2)
    assert(borrows[0]['user'] != "")
    assert(borrows[0]['token'] != "")
    assert(borrows[0]['amount'] > 0)
    assert(borrows[0]['timestamp'] > 0)


def test_borrow_aave_2_polygon_user():
    aave = Lending(protocol="Aave", chain="Polygon", version=2)
    borrows = aave.get_data_from_date_range(
        '11/05/2021 00:00:01', '12/05/2021 00:01:10', "borrow", "0x1fd51540eb1609f58499bddef0b3c345d5d92e3c")

    for borrow in borrows:
        assert(borrow['user'] == "0x1fd51540eb1609f58499bddef0b3c345d5d92e3c")


def test_borrow_compound_2_eth():
    compound = Lending(protocol="Compound", chain="Ethereum", version=2)
    borrows = compound.get_data_from_date_range(
        '11/05/2021 00:00:01', '11/05/2021 1:01:10', "borrow")

    assert(borrows[0]['tx_id'] != "")
    assert(borrows[0]['protocol'] == "Compound")
    assert(borrows[0]['chain'] == "Ethereum")
    assert(borrows[0]['version'] == 2)
    assert(borrows[0]['user'] != "")
    assert(borrows[0]['token'] != "")
    assert(float(borrows[0]['amount']) > 0)
    assert(borrows[0]['timestamp'] > 0)


def test_borrow_compound_2_eth_user():
    compound = Lending(protocol="Compound", chain="Ethereum", version=2)
    borrows = compound.get_data_from_date_range(
        '11/05/2021 00:00:01', '14/05/2021 00:01:10', "borrow", "0x162a7cec46225fb915dfd384ae049025dfcf4c10")

    for borrow in borrows:
        assert(borrow['user'] == "0x162a7cec46225fb915dfd384ae049025dfcf4c10")


def test_borrow_cream_2_eth():
    cream = Lending(protocol="Cream", chain="Ethereum", version=2)
    borrows = cream.get_data_from_date_range(
        '11/05/2021 00:00:01', '12/05/2021 11:54:10', "borrow")

    assert(borrows[0]['tx_id'] != "")
    assert(borrows[0]['protocol'] == "Cream")
    assert(borrows[0]['chain'] == "Ethereum")
    assert(borrows[0]['version'] == 2)
    assert(borrows[0]['user'] != "")
    assert(borrows[0]['token'] != "")
    assert(float(borrows[0]['amount']) > 0)
    assert(borrows[0]['timestamp'] > 0)


def test_borrow_cream_2_polygon():
    cream = Lending(protocol="Cream", chain="Polygon", version=2)
    borrows = cream.get_data_from_date_range(
        '23/09/2021 00:00:01', '24/09/2021 11:54:10', "borrow")

    assert(borrows[0]['tx_id'] != "")
    assert(borrows[0]['protocol'] == "Cream")
    assert(borrows[0]['chain'] == "Polygon")
    assert(borrows[0]['version'] == 2)
    assert(borrows[0]['user'] != "")
    assert(borrows[0]['token'] != "")
    assert(float(borrows[0]['amount']) > 0)
    assert(borrows[0]['timestamp'] > 0)

def test_borrow_cream_2_arbitrum():
    cream = Lending(protocol="Cream", chain="Arbitrum", version=2)
    borrows = cream.get_data_from_date_range(
        '23/09/2021 00:00:01', '25/09/2021 11:54:10', "borrow")

    assert(borrows[0]['tx_id'] != "")
    assert(borrows[0]['protocol'] == "Cream")
    assert(borrows[0]['chain'] == "Arbitrum")
    assert(borrows[0]['version'] == 2)
    assert(borrows[0]['user'] != "")
    assert(borrows[0]['token'] != "")
    assert(float(borrows[0]['amount']) > 0)
    assert(borrows[0]['timestamp'] > 0)


def test_borrow_cream_2_fantom():
    cream = Lending(protocol="Cream", chain="fantom", version=2)
    borrows = cream.get_data_from_date_range(
        '20/10/2021 00:00:01', '21/10/2021 00:00:00', "borrow")

    assert(borrows[0]['tx_id'] != "")
    assert(borrows[0]['protocol'] == "Cream")
    assert(borrows[0]['chain'] == "fantom")
    assert(borrows[0]['version'] == 2)
    assert(borrows[0]['user'] != "")
    assert(borrows[0]['token'] != "")
    assert(float(borrows[0]['amount']) > 0)
    assert(borrows[0]['timestamp'] > 0)


def test_borrow_cream_2_avalanche():
    cream = Lending(protocol="Cream", chain="avalanche", version=2)
    borrows = cream.get_data_from_date_range(
        '20/10/2021 00:00:01', '21/10/2021 00:00:00', "borrow")

    assert(borrows[0]['tx_id'] != "")
    assert(borrows[0]['protocol'] == "Cream")
    assert(borrows[0]['chain'] == "avalanche")
    assert(borrows[0]['version'] == 2)
    assert(borrows[0]['user'] != "")
    assert(borrows[0]['token'] != "")
    assert(float(borrows[0]['amount']) > 0)
    assert(borrows[0]['timestamp'] > 0)


def test_borrow_cream_2_eth_user():
    cream = Lending(protocol="Cream", chain="Ethereum", version=2)
    borrows = cream.get_data_from_date_range(
        '11/05/2021 00:00:01', '14/05/2021 11:54:10', "borrow", "0x1a32b5993f42e3df3e37cbce3cf270d44a91a960")

    for borrow in borrows:
        assert(borrow['user'] == "0x1a32b5993f42e3df3e37cbce3cf270d44a91a960")


def test_borrow_cream_2_bsc():
    cream = Lending(protocol="Cream", chain="bsc", version=2)
    borrows = cream.get_data_from_date_range(
        '08/05/2021 00:00:01', '12/05/2021 11:54:10', "borrow")

    assert(borrows[0]['tx_id'] != "")
    assert(borrows[0]['protocol'] == "Cream")
    assert(borrows[0]['chain'] == "bsc")
    assert(borrows[0]['version'] == 2)
    assert(borrows[0]['user'] != "")
    assert(borrows[0]['token'] != "")
    assert(float(borrows[0]['amount']) > 0)
    assert(borrows[0]['timestamp'] > 0)


def test_borrow_cream_2_bsc_user():
    cream = Lending(protocol="Cream", chain="bsc", version=2)
    borrows = cream.get_data_from_date_range(
        '08/05/2021 00:00:01', '14/05/2021 11:54:10', "borrow", "0x8a8aaaefbeee3fc7ca30a0f9a2c5c5ea0e83ebb0")

    for borrow in borrows:
        assert(borrow['user'] == "0x8a8aaaefbeee3fc7ca30a0f9a2c5c5ea0e83ebb0")



def test_borrow_kashi_1_eth():
    kashi = Lending(protocol="Kashi", chain="Ethereum", version=1)
    borrows = kashi.get_data_from_date_range(
        '30/09/2021 00:00:01', '30/09/2021 11:54:10', "borrow")

    assert(borrows[0]['tx_id'] != "")
    assert(borrows[0]['protocol'] == "Kashi")
    assert(borrows[0]['chain'] == "Ethereum")
    assert(borrows[0]['version'] == 1)
    assert(borrows[0]['user'] != "")
    assert(borrows[0]['token'] != "")
    assert(float(borrows[0]['amount']) > 0)
    assert(borrows[0]['timestamp'] > 0)


def test_borrow_kashi_1_eth_user():
    kashi = Lending(protocol="Kashi", chain="Ethereum", version=1)
    borrows = kashi.get_data_from_date_range(
        '20/09/2021 00:00:01', '30/09/2021 11:54:10', "borrow", "0xb715606b0482bd0ec5c54e8dc616e1deb59d5308")

    for borrow in borrows:
        assert(borrow['user'] == "0xb715606b0482bd0ec5c54e8dc616e1deb59d5308")


def test_borrow_kashi_1_polygon():
    kashi = Lending(protocol="Kashi", chain="Polygon", version=1)
    borrows = kashi.get_data_from_date_range(
        '30/09/2021 00:00:01', '30/09/2021 11:54:10', "borrow")

    assert(borrows[0]['tx_id'] != "")
    assert(borrows[0]['protocol'] == "Kashi")
    assert(borrows[0]['chain'] == "Polygon")
    assert(borrows[0]['version'] == 1)
    assert(borrows[0]['user'] != "")
    assert(borrows[0]['token'] != "")
    assert(float(borrows[0]['amount']) > 0)
    assert(borrows[0]['timestamp'] > 0)



from deficrawler.protocol_base import ProtocolBase


class Block(ProtocolBase):
    """
    Class to get data for the blockchain blocks
    """

    def __init__(self, protocol, chain, version):
        super().__init__(
            protocol=protocol,
            chain=chain,
            version=version
        )
        self.protocol_type = "blocks"

    def get_block_at_timestamp(self, timestamp):
        """
        Gets the block number for a specific timestamp
        """

        config = super().get_protocol_config('block')

        response_data = super().query_first_element(
            entity='block',
            timestamp=timestamp,
            aditional_filters=''
        )

        return super().map_data(
            response_data=response_data,
            config=config
        )

    def get_block_info(self, block_number):
        """
        Gets the block information related to a block number
        """

        config = super().get_protocol_config('block_info')

        block_name = self.mappings_file['entities']['block_info']['query']['params']['number']

        response_data = super().query_data_filtered(
            filters={block_name: str(block_number)},
            entity='block_info'
        )

        return super().map_data(
            response_data=response_data,
            config=config
        )

    def supported_entities(self):
        """
        Returns the supported entities for the protocol
        """
        return super().supported_entities(self.protocol_type)



from deficrawler.protocol_base import ProtocolBase

from datetime import datetime


class Dex(ProtocolBase):
    """
    Class to get data for Dexes protocols
    """

    def __init__(self, protocol, chain, version):
        super().__init__(
            protocol=protocol,
            chain=chain,
            version=version
        )
        self.protocol_type = "dex"

    def get_data_from_date_range(self, from_date, to_date, entity, pool=''):
        """
        Gets data for the specified entity in the from_data to to_date period.
        The entities are defined in the configuration of each protocol.
        """

        from_timestamp = int(
            datetime.strptime(from_date, '%d/%m/%Y %H:%M:%S').timestamp())

        to_timestamp = int(datetime.strptime(
            to_date, '%d/%m/%Y %H:%M:%S').timestamp())

        config = super().get_protocol_config(entity)

        pool_filter = ''
        if(pool != ''):
            pool_filter = {
                self.mappings_file['entities'][entity]['query']['params']['pool']: pool
            }

        response_data = super().query_data_from_date_range(
            from_timestamp=from_timestamp,
            to_timestamp=to_timestamp,
            entity=entity,
            aditional_filters=pool_filter
        )

        return super().map_data(
            response_data=response_data,
            config=config
        )

    def get_all_pools(self):
        """
        Returns all pools of the protocol
        """

        config = super().get_protocol_config('pool')

        response_data = super().query_data_parameter(
            entity='pool'
        )

        return super().map_data(
            response_data=response_data,
            config=config
        )

    def supported_entities(self):
        """
        Returns the supported entities for the protocol
        """
        return super().supported_entities(self.protocol_type)



from deficrawler.querys import Querys
from deficrawler.mappers import Mappers
from deficrawler.api_calls import get_data_from, get_data_parameter, get_data_filtered, get_first_element

import pkgutil
import json


class ProtocolBase:
    """
    Parent class to be inherit from Protocol and oracle
    """

    def __init__(self, protocol, chain, version):
        self.protocol = protocol
        self.query_from_timestamp = Querys.QUERY_FROM_TIMESTAMP
        self.query_all_elements = Querys.QUERY_ALL_ELEMENTS
        self.query_filter = Querys.QUERY_ELEMENT_FILTER
        self.query_first = Querys.QUERY_FIRST_ELEMENT
        self.mappings_file = self.__get_protocol_file(protocol, version, chain)
        self.chain = chain
        self.version = version
        self.endpoint = self.__get_protocol_endpoint(chain)
        self.global_config = self.__load_global_config()

    def query_data_from_date_range(self, from_timestamp, to_timestamp, entity, aditional_filters=''):
        """
        Gets all the data from the given entity and the specified period. One or more filters can be applied
        """

        return get_data_from(query_input=self.query_from_timestamp,
                             from_timestamp=from_timestamp,
                             to_timestamp=to_timestamp,
                             entity=entity,
                             mappings_file=self.mappings_file,
                             endpoint=self.endpoint,
                             aditional_filters=aditional_filters)

    def query_data_parameter(self, entity):
        """
        Gets all the existing data for the given entity
        """

        return get_data_parameter(query_input=self.query_all_elements,
                                  entity=entity,
                                  mappings_file=self.mappings_file,
                                  endpoint=self.endpoint)

    def query_data_filtered(self, entity, filters):
        """
        Gets all the existing data for the given entity with the specified filters
        """

        return get_data_filtered(query_input=self.query_filter,
                                 entity=entity,
                                 mappings_file=self.mappings_file,
                                 endpoint=self.endpoint,
                                 filters=filters)

    def query_first_element(self, entity, timestamp, aditional_filters, block=None):
        """
        Gets the first existing element for the given entity with the specified filters
        """
        return get_first_element(query_input=self.query_first,
                                 entity=entity,
                                 mappings_file=self.mappings_file,
                                 endpoint=self.endpoint,
                                 timestamp=timestamp,
                                 aditional_filters=aditional_filters,
                                 block=block)

    def map_data(self, response_data, config):
        """
        Maps the data from the subgraph data to the defined commom model in
        the json file
        """
        return Mappers.map_data(response_data=response_data,
                                protocol=self.protocol,
                                chain=self.chain,
                                version=self.version,
                                attributes=config['attributes'],
                                transformations=config['transformations'],
                                query_elements=config['query_elements'])

    def get_protocol_config(self, entity):
        """
        Gets the protocol config in the json file for the specified entity.
        If the entity not exists will raise an error.
        """

        self.__exists_entity(entity)

        query_elements = self.mappings_file['entities'][entity]['query']['extra_fields']
        query_elements.update(
            self.mappings_file['entities'][entity]['attributes'])

        return {
            'attributes': self.mappings_file['entities'][entity]['attributes'],
            'transformations': self.mappings_file['entities'][entity]['transformations'],
            'query_elements': query_elements,
        }

    def supported_entities(self, protocol_type):
        """
        Returns an array with the supported entities for each type of protocol
        """
        supported = []
        protocol_type_entities = self.global_config['supported_entities'][protocol_type]
        for attr, _ in self.mappings_file['entities'].items():
            if attr in protocol_type_entities:
                supported.append(attr)

        return supported

    def __get_protocol_file(self, protocol, version, chain):
        """
        Gets the json file for the specified protocol and the specified version.
        If the protocol does not exits at this version will raise an error.
        """
        try:
            config_file = pkgutil.get_data(
                'deficrawler.config',
                protocol.lower() + "-" + str(version) + "-" + chain.lower() + ".json"
            )

        except FileNotFoundError:
            try:
                config_file = pkgutil.get_data(
                    'deficrawler.config',
                    protocol.lower() + "-" + str(version) + ".json"
                )
            except:
                raise Exception('Protocol ' + protocol +
                                ' version ' + str(version) + ' not supported')
        except:
            raise

        return json.loads(config_file.decode())

    def __get_protocol_endpoint(self, chain):
        """
        Gets the endpoint for the specified chain.
        If the chain does not exits in the config file will raise an error.
        """
        try:
            return self.mappings_file['protocol']['endpoint'][chain.lower()]
        except KeyError:
            raise Exception('Protocol ' + self.protocol +
                            ' chain ' + chain + ' not supported')
        except:
            raise

    def __exists_entity(self, entity):
        """
        Checks if the given entity exists in the entities config.
        In not exists will raise an error.
        """
        if entity in self.mappings_file['entities']:
            return True
        else:
            raise Exception('Entity ' + entity +
                            ' not supported for protocol ' + self.protocol)

    def __load_global_config(self):
        """
        Loads the global config file
        """
        try:
            global_config_file = pkgutil.get_data(
                "deficrawler.config",
                "global.json"
            )

        except FileNotFoundError:
            raise Exception('Error loading global config')
        except:
            raise

        return json.loads(global_config_file.decode())



from deficrawler.dex import Dex


def test_mint_uniswap_2_eth():
    uniswap = Dex(protocol="Uniswap", chain="Ethereum", version=2)
    mints = uniswap.get_data_from_date_range(
        '04/05/2021 00:00:00', '04/05/2021 00:30:30', "mint")

    assert(mints[0]['tx_id'] != "")
    assert(mints[0]['protocol'] == "Uniswap")
    assert(mints[0]['chain'] == "Ethereum")
    assert(mints[0]['version'] == 2)
    assert(mints[0]['user'] != "")
    assert(mints[0]['token0'] != "")
    assert(mints[0]['token1'] != "")
    assert(mints[0]['pool'] != "")
    assert(float(mints[0]['amount0']) > 0)
    assert(float(mints[0]['amount1']) > 0)
    assert(float(mints[0]['timestamp']) > 0)


def test_mint_uniswap_3_eth():
    uniswap = Dex(protocol="Uniswap", chain="Ethereum", version=3)
    mints = uniswap.get_data_from_date_range(
        '30/09/2021 00:00:00', '30/09/2021 02:00:30', "mint")

    assert(mints[0]['tx_id'] != "")
    assert(mints[0]['protocol'] == "Uniswap")
    assert(mints[0]['chain'] == "Ethereum")
    assert(mints[0]['version'] == 3)
    assert(mints[0]['user'] != "")
    assert(mints[0]['token0'] != "")
    assert(mints[0]['token1'] != "")
    assert(mints[0]['pool'] != "")
    assert(float(mints[0]['amount0']) > 0)
    assert(float(mints[0]['amount1']) > 0)
    assert(float(mints[0]['timestamp']) > 0)



# def test_mint_ubeswap_2_celo():
#     ubeswap = Dex(protocol="Ubeswap", chain="Celo", version=1)
#     mints = ubeswap.get_data_from_date_range(
#         '30/09/2021 00:00:00', '30/09/2021 01:00:30', "mint")

#     assert(mints[0]['tx_id'] != "")
#     assert(mints[0]['protocol'] == "Ubeswap")
#     assert(mints[0]['chain'] == "Celo")
#     assert(mints[0]['version'] == 1)
#     assert(mints[0]['user'] != "")
#     assert(mints[0]['token0'] != "")
#     assert(mints[0]['token1'] != "")
#     assert(mints[0]['pool'] != "")
#     assert(float(mints[0]['amount0']) > 0)
#     assert(float(mints[0]['amount1']) > 0)
#     assert(float(mints[0]['timestamp']) > 0)


def test_mint_sushi_2_eth():
    sushi = Dex(protocol="Sushiswap", chain="Ethereum", version=1)
    mints = sushi.get_data_from_date_range(
        '30/09/2021 00:00:00', '30/09/2021 01:00:30', "mint")

    assert(mints[0]['tx_id'] != "")
    assert(mints[0]['protocol'] == "Sushiswap")
    assert(mints[0]['chain'] == "Ethereum")
    assert(mints[0]['version'] == 1)
    assert(mints[0]['user'] != "")
    assert(mints[0]['token0'] != "")
    assert(mints[0]['token1'] != "")
    assert(mints[0]['pool'] != "")
    assert(float(mints[0]['amount0']) > 0)
    assert(float(mints[0]['amount1']) > 0)
    assert(float(mints[0]['timestamp']) > 0)

def test_mint_sushi_2_bsc():
    sushi = Dex(protocol="Sushiswap", chain="bsc", version=1)
    mints = sushi.get_data_from_date_range(
        '30/09/2021 00:00:00', '30/09/2021 02:00:30', "mint")

    assert(mints[0]['tx_id'] != "")
    assert(mints[0]['protocol'] == "Sushiswap")
    assert(mints[0]['chain'] == "bsc")
    assert(mints[0]['version'] == 1)
    assert(mints[0]['user'] != "")
    assert(mints[0]['token0'] != "")
    assert(mints[0]['token1'] != "")
    assert(mints[0]['pool'] != "")
    assert(float(mints[0]['amount0']) > 0)
    assert(float(mints[0]['amount1']) > 0)
    assert(float(mints[0]['timestamp']) > 0)


def test_mint_sushi_2_polygon():
    sushi = Dex(protocol="Sushiswap", chain="polygon", version=1)
    mints = sushi.get_data_from_date_range(
        '30/09/2021 00:00:00', '30/09/2021 01:00:30', "mint")

    assert(mints[0]['tx_id'] != "")
    assert(mints[0]['protocol'] == "Sushiswap")
    assert(mints[0]['chain'] == "polygon")
    assert(mints[0]['version'] == 1)
    assert(mints[0]['user'] != "")
    assert(mints[0]['token0'] != "")
    assert(mints[0]['token1'] != "")
    assert(mints[0]['pool'] != "")
    assert(float(mints[0]['amount0']) > 0)
    assert(float(mints[0]['amount1']) > 0)
    assert(float(mints[0]['timestamp']) > 0)

def test_mint_sushi_2_fantom():
    sushi = Dex(protocol="Sushiswap", chain="fantom", version=1)
    mints = sushi.get_data_from_date_range(
        '30/09/2021 00:00:00', '30/09/2021 01:00:30', "mint")

    assert(mints[0]['tx_id'] != "")
    assert(mints[0]['protocol'] == "Sushiswap")
    assert(mints[0]['chain'] == "fantom")
    assert(mints[0]['version'] == 1)
    assert(mints[0]['user'] != "")
    assert(mints[0]['token0'] != "")
    assert(mints[0]['token1'] != "")
    assert(mints[0]['pool'] != "")
    assert(float(mints[0]['amount0']) > 0)
    assert(float(mints[0]['amount1']) > 0)
    assert(float(mints[0]['timestamp']) > 0)

def test_mint_sushi_1_celo():
    sushi = Dex(protocol="Sushiswap", chain="celo", version=1)
    mints = sushi.get_data_from_date_range(
        '17/10/2021 00:00:00', '17/10/2021 03:00:30', "mint")

    assert(mints[0]['tx_id'] != "")
    assert(mints[0]['protocol'] == "Sushiswap")
    assert(mints[0]['chain'] == "celo")
    assert(mints[0]['version'] == 1)
    assert(mints[0]['user'] != "")
    assert(mints[0]['token0'] != "")
    assert(mints[0]['token1'] != "")
    assert(mints[0]['pool'] != "")
    assert(float(mints[0]['amount0']) > 0)
    assert(float(mints[0]['amount1']) > 0)
    assert(float(mints[0]['timestamp']) > 0)


def test_mint_pancakeswap_2_bsc():
    pancakeswap = Dex(protocol="Pancakeswap", chain="bsc", version=2)
    mints = pancakeswap.get_data_from_date_range(
        '30/09/2021 00:00:00', '30/09/2021 23:00:30', "mint")

    assert(mints[0]['tx_id'] != "")
    assert(mints[0]['protocol'] == "Pancakeswap")
    assert(mints[0]['chain'] == "bsc")
    assert(mints[0]['version'] == 2)
    assert(mints[0]['user'] != "")
    assert(mints[0]['token0'] != "")
    assert(mints[0]['token1'] != "")
    assert(mints[0]['pool'] != "")
    assert(float(mints[0]['amount0']) > 0)
    assert(float(mints[0]['amount1']) > 0)
    assert(float(mints[0]['timestamp']) > 0)


def test_mint_quickswap_1_polygon():
    quickswap = Dex(protocol="quickswap", chain="polygon", version=1)
    mints = quickswap.get_data_from_date_range(
        '30/09/2021 00:00:00', '30/09/2021 01:00:30', "mint")

    assert(mints[0]['tx_id'] != "")
    assert(mints[0]['protocol'] == "quickswap")
    assert(mints[0]['chain'] == "polygon")
    assert(mints[0]['version'] == 1)
    assert(mints[0]['user'] != "")
    assert(mints[0]['token0'] != "")
    assert(mints[0]['token1'] != "")
    assert(mints[0]['pool'] != "")
    assert(float(mints[0]['amount0']) > 0)
    assert(float(mints[0]['amount1']) > 0)
    assert(float(mints[0]['timestamp']) > 0)


def test_mint_pangolin_1_avalanche():
    pangolin = Dex(protocol="pangolin", chain="avalanche", version=1)
    mints = pangolin.get_data_from_date_range(
        '30/09/2021 00:00:00', '30/09/2021 01:00:30', "mint")

    assert(mints[0]['tx_id'] != "")
    assert(mints[0]['protocol'] == "pangolin")
    assert(mints[0]['chain'] == "avalanche")
    assert(mints[0]['version'] == 1)
    assert(mints[0]['user'] != "")
    assert(mints[0]['token0'] != "")
    assert(mints[0]['token1'] != "")
    assert(mints[0]['pool'] != "")
    assert(float(mints[0]['amount0']) > 0)
    assert(float(mints[0]['amount1']) > 0)
    assert(float(mints[0]['timestamp']) > 0)


def test_mint_traderjoe_1_avalanche():
    traderjoe = Dex(protocol="traderjoe", chain="avalanche", version=1)
    mints = traderjoe.get_data_from_date_range(
        '30/09/2021 00:00:00', '30/09/2021 01:00:30', "mint")

    assert(mints[0]['tx_id'] != "")
    assert(mints[0]['protocol'] == "traderjoe")
    assert(mints[0]['chain'] == "avalanche")
    assert(mints[0]['version'] == 1)
    assert(mints[0]['user'] != "")
    assert(mints[0]['token0'] != "")
    assert(mints[0]['token1'] != "")
    assert(mints[0]['pool'] != "")
    assert(float(mints[0]['amount0']) > 0)
    assert(float(mints[0]['amount1']) > 0)
    assert(float(mints[0]['timestamp']) > 0)


from deficrawler.utils import format_attribute, format_element, get_attributes, get_filters

import pkgutil
import json


def test_get_filters():
    filters_empty = get_filters('')
    assert(filters_empty == '')

    one_filter = get_filters({"user": "account"})
    assert(one_filter == "user:\"account\"\n")

    several_filtes = get_filters({"user": "account", "pair": "assetPair"})
    assert(several_filtes == "user:\"account\"\npair:\"assetPair\"\n")


def test_format_element():
    one_element = ["id"]
    one_formatted = format_element(one_element)
    assert(one_formatted == "id")

    two_elements = ["user", "id"]
    two_formatted = format_element(two_elements)
    assert(two_formatted == "user{id}")

    three_elements = ["pair", "token", "symbol"]
    three_formatted = format_element(three_elements)
    assert(three_formatted == "pair{token{symbol}}")


def test_format_attribute():
    one_attribute = ["sender"]
    one_formatted = format_attribute(one_attribute)
    assert(one_formatted == "sender")

    two_attribute = [
        [
            "pair",
            "token0",
            "symbol"
        ],
        [
            "pair",
            "token1",
            "symbol"
        ]
    ]

    two_formatted = format_attribute(two_attribute)
    assert(two_formatted == "pair{token0{symbol}} pair{token1{symbol}} ")


def test_get_attributes():
    config_file = pkgutil.get_data(
        'deficrawler.config',
        'aave' + "-" + str('2') + ".json"
    )

    map_file = json.loads(config_file.decode())
    borrow_attr = get_attributes('borrow', map_file)
    print(borrow_attr == "reserve{decimals} id user{id} reserve{symbol} amount timestamp ")

    config_file = pkgutil.get_data(
        'deficrawler.config',
        'uniswap' + "-" + str('2') + ".json"
    )

    map_file = json.loads(config_file.decode())
    swap_attr = get_attributes('swap', map_file)
    assert(swap_attr =="sender transaction{id} pair{token0{symbol}} pair{token1{symbol}}  pair{token0{symbol}} pair{token1{symbol}}  amount0In amount1In  amount0Out amount1Out  pair{id} timestamp ")




from deficrawler.transformer import Transformer


class Mappers:
    """
    Class to map the data from the subgraph data to the commom model defined in the json file.
    For each field applies the transformaton (if needed) and retuns the entity with all
    the fields.
    """

    @staticmethod
    def map_data(response_data, protocol, chain, version, attributes, transformations, query_elements):
        list_elements = []

        transformer = Transformer()

        for ele in response_data:
            element = {}
            for common_field, protocol_field in attributes.items():
                element[common_field] = (
                    transformer.transform(
                        element=ele,
                        common_field=common_field,
                        protocol_field=protocol_field,
                        transformations=transformations,
                        query_elements=query_elements
                    )
                )
                element['protocol'] = protocol
                element['chain'] = chain
                element['version'] = version
            list_elements.append(element)

        return list_elements



import dict_digger


class Transformer:
    """
    This class applies the transformations defined in the json config.
    Apply the specified function to transform the data from the subgraph to
    the common model format.
    """

    def __init__(self):
        """
        Constructor. Defines the dictionary that will be use to link the function name specified
        in the config file to the class function to apply.
        """
        self.transformers = {
            "decimals": self.transform_decimals,
            "principal_decimals": self.transform_principal_decimals,
            "collateral_decimals": self.transform_collateral_decimals,
            "from_token": self.from_token_selection,
            "to_token": self.to_token_selection,
            "from_token_amount": self.from_token_amount_selection,
            "to_token_amount": self.to_token_amount_selection,
            "from_token_v3": self.from_token_selection_v3,
            "to_token_v3": self.to_token_selection_v3,
            "from_token_amount_v3": self.from_token_amount_selection_v3,
            "to_token_amount_v3": self.to_token_amount_selection_v3,
            "array_length": self.array_length,
            "remove_token_prefix": self.remove_token_prefix,
            "tx_id_colon": self.tx_id_colon,
            "tx_id_hyphen": self.tx_id_hyphen,
            "chainlink_prices": self.chainlink_prices,
            "concat_symbols": self.concat_symbols,
            "concat_list_symbols": self.concat_list_symbols,
            "rates_units_aave": self.rates_units_aave,
            "rates_units_comp": self.rates_units_comp,
            "rates_na": self.rates_na,
            "to_int": self.to_int,
            "na": self.na
        }

    def transform(self, element, common_field, protocol_field, transformations, query_elements):
        """
        Main function, is called from the mapper, if the field to map has a transformation function
        contained in the class dictionary, the specified function will be applied, in other case
        the value will be returned without modification.
        """
        if common_field in transformations:
            type_transformer = transformations[common_field]
            return self.transformers[type_transformer](common_field, element, protocol_field, query_elements)
        else:
            return dict_digger.dig(
                element,
                *protocol_field)

    def transform_decimals_field(self, common_field, element, protocol_field, query_elements, field):
        """
        Transform the decimals of the number, dividing the field for the decimals value in the subgraph response
        """
        decimals_field = query_elements[field]
        dec_value = dict_digger.dig(
            element,
            *decimals_field)

        protocol_amount = dict_digger.dig(
            element,
            *protocol_field)

        return float(protocol_amount) / 10 ** float(dec_value)

    def transform_decimals(self, common_field, element, protocol_field, query_elements):
        """
        Use the transform_decimals_field function to transform the decimals in the `decimals` field
        """
        return self.transform_decimals_field(common_field, element, protocol_field, query_elements, 'decimals')

    def transform_principal_decimals(self, common_field, element, protocol_field, query_elements):
        """
        Use the transform_decimals_field function to transform the decimals in the `principal_decimals` field
        """
        return self.transform_decimals_field(common_field, element, protocol_field, query_elements, 'principal_decimals')

    def transform_collateral_decimals(self, common_field, element, protocol_field, query_elements):
        """
        Use the transform_decimals_field function to transform the decimals in the `collateral_decimals` field
        """
        return self.transform_decimals_field(common_field, element, protocol_field, query_elements, 'collateral_decimals')

    def from_token_selection(self, common_field, element, protocol_field, query_elements):
        """
        Gets the from token in a swap, gets the field with amount != 0
        """
        amount_0_in = dict_digger.dig(
            element,
            *query_elements['from_token_amount'][0])

        index = 0 if float(amount_0_in) > 0 else 1

        return dict_digger.dig(
            element,
            *query_elements['from_token'][index])

    def to_token_selection(self, common_field, element, protocol_field, query_elements):
        """
        Gets the to token in a swap, gets the field with amount != 0
        """
        amount_0_out = dict_digger.dig(
            element,
            *query_elements['to_token_amount'][0])

        index = 0 if float(amount_0_out) > 0 else 1

        return dict_digger.dig(
            element,
            *query_elements['to_token'][index])

    def from_token_amount_selection(self, common_field, element, protocol_field, query_elements):
        """
        Gets the from token amount in a swap, gets the field with amount != 0
        """
        amount_0_in = dict_digger.dig(
            element,
            *query_elements['from_token_amount'][0])

        return amount_0_in if float(amount_0_in) > 0 else dict_digger.dig(
            element,
            *query_elements['from_token_amount'][1])

    def to_token_amount_selection(self, common_field, element, protocol_field, query_elements):
        """
        Gets the to token amount in a swap, gets the field with amount != 0
        """
        amount_0_out = dict_digger.dig(
            element,
            *query_elements['to_token_amount'][0])

        return amount_0_out if float(amount_0_out) > 0 else dict_digger.dig(
            element,
            *query_elements['to_token_amount'][1])

    def array_length(self, common_field, element, protocol_field, query_elements):
        """
        Returns the length of a field of array type
        """
        array_field = dict_digger.dig(
            element,
            query_elements[common_field][0])

        return len(array_field)

    def remove_token_prefix(self, common_field, element, protocol_field, query_elements):
        """
        Removes the token prefix of the token. Usefull to remove cTOkens or aTokens
        """
        field = dict_digger.dig(
            element,
            *query_elements[common_field])

        return field[1:]

    def tx_id_colon(self, common_field, element, protocol_field, query_elements):
        """
        Removes the trailing data afeter the : in the transaction id field
        """
        field = dict_digger.dig(
            element,
            *query_elements[common_field])

        return field.split(':')[0]

    def tx_id_hyphen(self, common_field, element, protocol_field, query_elements):
        """
        Removes the trailing data afeter the - in the transaction id field
        """
        field = dict_digger.dig(
            element,
            *query_elements[common_field])

        return field.split('-')[0]

    def chainlink_prices(self, common_field, element, protocol_field, query_elements):
        """
        Returns the chainlink prices in the correct format, if ETH pair are 18 decimals
        in other case 8 decimals
        """
        price = dict_digger.dig(
            element,
            *query_elements[common_field])

        pair = dict_digger.dig(
            element,
            *['assetPair', 'id'])

        pair_decimals = 8
        tokens = pair.split('/')
        if len(tokens) > 1:
            if(tokens[1] == 'ETH'):
                pair_decimals = 18

        return float(price) / 10 ** pair_decimals

    def concat_symbols(self, common_field, element, protocol_field, query_elements):
        """
        Return the list of symbols of the pool concatenated
        """
        tokens_concat = ''
        tokens_list = query_elements[common_field]

        for index, token_path in enumerate(tokens_list):
            token = dict_digger.dig(
                element,
                *token_path)
            tokens_concat += token
            if(index + 1) < len(tokens_list):
                tokens_concat += '/'

        return tokens_concat

    def concat_list_symbols(self, common_field, element, protocol_field, query_elements):
        """
        Return the list of symbols of the pool concatenated
        """
        tokens_concat = ''

        tokens_list = dict_digger.dig(
            element,
            query_elements[common_field][0])

        field_name = query_elements[common_field][1]

        for index, token in enumerate(tokens_list):
            tokens_concat += token[field_name]
            if(index + 1) < len(tokens_list):
                tokens_concat += '/'

        return tokens_concat

    def rates_units_aave(self, common_field, element, protocol_field, query_elements):
        """
        Return the aave rates units converted
        """

        rate = dict_digger.dig(
            element,
            *query_elements[common_field])

        return float(rate)/1e25

    def rates_units_comp(self, common_field, element, protocol_field, query_elements):
        """
        Return the comp rates units converted
        """

        rate = dict_digger.dig(
            element,
            *query_elements[common_field])

        return float(rate) * 1e2

    def rates_na(self, common_field, element, protocol_field, query_elements):
        """
        Return zero to not applicable rate value
        """

        return 
        
    def from_token_selection_v3(self, common_field, element, protocol_field, query_elements):
        """
        Gets the from token in a swap for uniswap v3, gets the field with amount positive
        """
        amount_0 = dict_digger.dig(
            element,
            *query_elements['from_token_amount'][0])

        index = 0 if (amount_0.find('-') == -1) else 1

        return dict_digger.dig(
            element,
            *query_elements['from_token'][index])

    def to_token_selection_v3(self, common_field, element, protocol_field, query_elements):
        """
        Gets the to token in a swap for uniswap v3, gets the field with amount negative
        """
        amount_0 = dict_digger.dig(
            element,
            *query_elements['to_token_amount'][0])

        index = 0 if (amount_0.find('-') != -1) else 1

        return dict_digger.dig(
            element,
            *query_elements['to_token'][index])

    def from_token_amount_selection_v3(self, common_field, element, protocol_field, query_elements):
        """
        Gets the from token amount in a swap, gets the field with amount positive
        """
        amount_0 = dict_digger.dig(
            element,
            *query_elements['from_token_amount'][0])

        index = 0 if (amount_0.find('-') == -1) else 1

        amount = dict_digger.dig(
            element,
            *query_elements['from_token_amount'][index])

        return amount.replace("-", "")

    def to_token_amount_selection_v3(self, common_field, element, protocol_field, query_elements):
        """
        Gets the to token amount in a swap, gets the field with amount negative
        """
        amount_0 = dict_digger.dig(
            element,
            *query_elements['from_token_amount'][0])

        index = 0 if (amount_0.find('-') != -1) else 1

        amount = dict_digger.dig(
            element,
            *query_elements['from_token_amount'][index])

        return amount.replace("-", "")


    def to_int(self, common_field, element, protocol_field, query_elements):
        """
        Transform the value to integer
        """
        ele = dict_digger.dig(
            element,
            *query_elements[common_field])

        return int(ele)

    def na(self, common_field, element, protocol_field, query_elements):
        """
        Returns NA for non applicable field
        """
        return 'NA'


from deficrawler.lending import Lending


def test_liquidation_aave_2_eth():
    aave = Lending(protocol="Aave", chain="Ethereum", version=2)
    liquidations = aave.get_data_from_date_range(
        '21/04/2021 05:20:01', '22/04/2021 06:22:01', "liquidation")
    assert(liquidations[0]['tx_id'] != "")
    assert(liquidations[0]['protocol'] == "Aave")
    assert(liquidations[0]['chain'] == "Ethereum")
    assert(liquidations[0]['version'] == 2)
    assert(liquidations[0]['user'] != "")
    assert(liquidations[0]['token_principal'] != "")
    assert(liquidations[0]['token_collateral'] != "")
    assert(liquidations[0]['amount_principal'] > 0)
    assert(liquidations[0]['amount_collateral'] > 0)
    assert(liquidations[0]['liquidator'] != "")
    assert(liquidations[0]['timestamp'] > 0)


def test_liquidation_aave_2_eth_user():
    aave = Lending(protocol="Aave", chain="Ethereum", version=2)
    liquidations = aave.get_data_from_date_range(
        '21/04/2021 05:20:01', '24/04/2021 06:22:01', "liquidation", "0xcfd873f19a86b84cfc4916e8623f2486dc83d792")

    for liquidation in liquidations:
        assert(liquidation['user'] ==
               "0xcfd873f19a86b84cfc4916e8623f2486dc83d792")


def test_liquidation_aave_2_polygon():
    aave = Lending(protocol="Aave", chain="Polygon", version=2)
    liquidations = aave.get_data_from_date_range(
        '10/05/2021 00:00:01', '11/05/2021 00:01:10', "liquidation")

    assert(liquidations[0]['tx_id'] != "")
    assert(liquidations[0]['protocol'] == "Aave")
    assert(liquidations[0]['chain'] == "Polygon")
    assert(liquidations[0]['version'] == 2)
    assert(liquidations[0]['user'] != "")
    assert(liquidations[0]['token_principal'] != "")
    assert(liquidations[0]['token_collateral'] != "")
    assert(liquidations[0]['amount_principal'] > 0)
    assert(liquidations[0]['amount_collateral'] > 0)
    assert(liquidations[0]['liquidator'] != "")
    assert(liquidations[0]['timestamp'] > 0)

def test_liquidation_aave_2_avalanche():
    aave = Lending(protocol="Aave", chain="Avalanche", version=2)
    liquidations = aave.get_data_from_date_range(
        '10/10/2021 00:00:01', '18/10/2021 00:01:10', "liquidation")

    assert(liquidations[0]['tx_id'] != "")
    assert(liquidations[0]['protocol'] == "Aave")
    assert(liquidations[0]['chain'] == "Avalanche")
    assert(liquidations[0]['version'] == 2)
    assert(liquidations[0]['user'] != "")
    assert(liquidations[0]['token_principal'] != "")
    assert(liquidations[0]['token_collateral'] != "")
    assert(liquidations[0]['amount_principal'] > 0)
    assert(liquidations[0]['amount_collateral'] > 0)
    assert(liquidations[0]['liquidator'] != "")
    assert(liquidations[0]['timestamp'] > 0)


def test_liquidation_aave_2_polygon_user():
    aave = Lending(protocol="Aave", chain="Polygon", version=2)
    liquidations = aave.get_data_from_date_range(
        '10/05/2021 00:00:01', '14/05/2021 00:01:10', "liquidation", "0x573bcd1d82b5bb799c5340e8f7077a4676f95097")

    for liquidation in liquidations:
        assert(liquidation['user'] ==
               "0x573bcd1d82b5bb799c5340e8f7077a4676f95097")


def test_liquidation_compound_2_eth():
    compound = Lending(protocol="Compound", chain="Ethereum", version=2)
    liquidations = compound.get_data_from_date_range(
        '09/05/2021 00:00:01', '11/05/2021 00:01:10', "liquidation")

    assert(liquidations[0]['tx_id'] != "")
    assert(liquidations[0]['protocol'] == "Compound")
    assert(liquidations[0]['chain'] == "Ethereum")
    assert(liquidations[0]['version'] == 2)
    assert(liquidations[0]['user'] != "")
    assert(liquidations[0]['token_principal'] != "")
    assert(liquidations[0]['token_collateral'] != "")
    assert(float(liquidations[0]['amount_principal']) > 0)
    assert(float(liquidations[0]['amount_collateral']) > 0)
    assert(liquidations[0]['liquidator'] != "")
    assert(liquidations[0]['timestamp'] > 0)


def test_liquidation_compound_2_eth_user():
    compound = Lending(protocol="Compound", chain="Ethereum", version=2)
    liquidations = compound.get_data_from_date_range(
        '09/05/2021 00:00:01', '14/05/2021 00:01:10', "liquidation", "0xa507b355d6288a232ac692dad36af80ff1eba062")

    for liquidation in liquidations:
        assert(liquidation['user'] ==
               "0xa507b355d6288a232ac692dad36af80ff1eba062")


def test_liquidation_cream_2_eth():
    cream = Lending(protocol="Cream", chain="Ethereum", version=2)
    liquidations = cream.get_data_from_date_range(
        '01/05/2021 00:00:01', '12/05/2021 11:54:10', "liquidation")

    assert(liquidations[0]['tx_id'] != "")
    assert(liquidations[0]['protocol'] == "Cream")
    assert(liquidations[0]['chain'] == "Ethereum")
    assert(liquidations[0]['version'] == 2)
    assert(liquidations[0]['user'] != "")
    assert(liquidations[0]['token_principal'] != "")
    assert(liquidations[0]['token_collateral'] != "")
    assert(float(liquidations[0]['amount_principal']) > 0)
    assert(float(liquidations[0]['amount_collateral']) > 0)
    assert(liquidations[0]['liquidator'] != "")
    assert(liquidations[0]['timestamp'] > 0)



def test_liquidation_cream_2_polygon():
    cream = Lending(protocol="Cream", chain="Polygon", version=2)
    liquidations = cream.get_data_from_date_range(
        '25/09/2021 00:00:01', '26/09/2021 11:54:10', "liquidation")

    assert(liquidations[0]['tx_id'] != "")
    assert(liquidations[0]['protocol'] == "Cream")
    assert(liquidations[0]['chain'] == "Polygon")
    assert(liquidations[0]['version'] == 2)
    assert(liquidations[0]['user'] != "")
    assert(liquidations[0]['token_principal'] != "")
    assert(liquidations[0]['token_collateral'] != "")
    assert(float(liquidations[0]['amount_principal']) > 0)
    assert(float(liquidations[0]['amount_collateral']) > 0)
    assert(liquidations[0]['liquidator'] != "")
    assert(liquidations[0]['timestamp'] > 0)



def test_liquidation_cream_2_eth_user():
    cream = Lending(protocol="Cream", chain="Ethereum", version=2)
    liquidations = cream.get_data_from_date_range(
        '01/05/2021 00:00:01', '14/05/2021 11:54:10', "liquidation","0xcb774b92587f1c19e960eaeb6902d97e2cabd6be")

    for liquidation in liquidations:
        assert(liquidation['user'] ==
               "0xcb774b92587f1c19e960eaeb6902d97e2cabd6be")


def test_liquidation_cream_2_bsc():
    cream = Lending(protocol="Cream", chain="bsc", version=2)
    liquidations = cream.get_data_from_date_range(
        '01/05/2021 00:00:01', '12/05/2021 11:54:10', "liquidation")

    assert(liquidations[0]['tx_id'] != "")
    assert(liquidations[0]['protocol'] == "Cream")
    assert(liquidations[0]['chain'] == "bsc")
    assert(liquidations[0]['version'] == 2)
    assert(liquidations[0]['user'] != "")
    assert(liquidations[0]['token_principal'] != "")
    assert(liquidations[0]['token_collateral'] != "")
    assert(float(liquidations[0]['amount_principal']) > 0)
    assert(float(liquidations[0]['amount_collateral']) > 0)
    assert(liquidations[0]['liquidator'] != "")
    assert(liquidations[0]['timestamp'] > 0)



def test_liquidation_cream_2_bsc_user():
    cream = Lending(protocol="Cream", chain="bsc", version=2)
    liquidations = cream.get_data_from_date_range(
        '01/05/2021 00:00:01', '14/05/2021 11:54:10', "liquidation", "0x0825602e9d177b51af7d79acbdc68a746bc41ce4")

    for liquidation in liquidations:
        assert(liquidation['user'] ==
               "0x0825602e9d177b51af7d79acbdc68a746bc41ce4")



from deficrawler.lending import Lending


def test_protocol_entities():
    aave = Lending(protocol="Aave", chain="Ethereum", version=2)
    compound = Lending(protocol="Compound", chain="Ethereum", version=2)

    assert(aave.supported_entities() == [
           'borrow', 'deposit', 'liquidation', 'repay'])
    assert(compound.supported_entities() == [
          'borrow', 'deposit', 'liquidation', 'repay'])



from deficrawler.protocol_base import ProtocolBase

from datetime import datetime


class Oracle(ProtocolBase):
    """
    Oracle class to get prices
    """

    def __init__(self, protocol, chain, version):
        super().__init__(
            protocol=protocol,
            chain=chain,
            version=version
        )

    def get_all_pairs(self):
        """
        Returns all the existing prices for the oracle.
        """

        config = super().get_protocol_config('pair')

        response_data = super().query_data_filtered(
            entity='pair',
            filters=''
        )

        return super().map_data(
            response_data=response_data,
            config=config
        )

    def get_price_from_date_range(self, from_date, to_date, pair):
        """
        Returns the prices series for the specified pair in the given period
        """

        from_timestamp = int(
            datetime.strptime(from_date, '%d/%m/%Y %H:%M:%S').timestamp())

        to_timestamp = int(datetime.strptime(
            to_date, '%d/%m/%Y %H:%M:%S').timestamp())

        config = super().get_protocol_config('price')

        pair_name = self.mappings_file['entities']['price']['query']['params']['pair']

        response_data = super().query_data_from_date_range(
            from_timestamp=from_timestamp,
            to_timestamp=to_timestamp,
            entity='price',
            aditional_filters={pair_name: pair}
        )

        return super().map_data(
            response_data=response_data,
            config=config
        )

    def get_price_at_timestamp(self, timestamp, pair):
        """
        Returns the prices for the specified pair in the given timestamp
        """

        config = super().get_protocol_config('price')

        pair_name = self.mappings_file['entities']['price']['query']['params']['pair']

        response_data = super().query_first_element(
            aditional_filters={pair_name: pair},
            entity='price',
            timestamp=timestamp
        )

        prices = super().map_data(
            response_data=response_data,
            config=config
        )

        if len(prices) > 0:
            return prices[0]['price']
        else:
            raise Exception(
                "Price not found for the specified pair at the timestamp")



from deficrawler.block import Block
from deficrawler.protocol_base import ProtocolBase

from datetime import datetime


class Lending(ProtocolBase):
    """
    Class to get data for Lending protocols
    """

    def __init__(self, protocol, chain, version):
        super().__init__(
            protocol=protocol,
            chain=chain,
            version=version
        )
        self.protocol_type = "lending"

    def get_data_from_date_range(self, from_date, to_date, entity, user=''):
        """
        Gets data for the specified entity in the from_data to to_date period.
        The entities are defined in the configuration of each protocol.
        """

        from_timestamp = int(
            datetime.strptime(from_date, '%d/%m/%Y %H:%M:%S').timestamp())

        to_timestamp = int(datetime.strptime(
            to_date, '%d/%m/%Y %H:%M:%S').timestamp())

        config = super().get_protocol_config(entity)

        aditional_filters = self.__get_aditional_filters__(
            user=user, entity=entity
        )

        response_data = super().query_data_from_date_range(
            from_timestamp=from_timestamp,
            to_timestamp=to_timestamp,
            entity=entity,
            aditional_filters=aditional_filters
        )

        return super().map_data(
            response_data=response_data,
            config=config
        )

    def get_rates_from_date_range(self, from_date, to_date, entity, asset):
        """
        Gets rates data for the specified entity in the from_data to to_date period.
        """

        from_timestamp = int(
            datetime.strptime(from_date, '%d/%m/%Y %H:%M:%S').timestamp())

        to_timestamp = int(datetime.strptime(
            to_date, '%d/%m/%Y %H:%M:%S').timestamp())

        config = super().get_protocol_config(entity)

        asset_filter = {
            self.mappings_file['entities'][entity]['query']['params']['asset']: asset
        }

        filter_by_block = 'block' in self.mappings_file['entities'][entity]['query']['params']

        response_data = self.__get_data_from_blocks__(
            from_timestamp=from_timestamp,
            to_timestamp=to_timestamp,
            entity=entity,
            aditional_filters=asset_filter
        ) if filter_by_block else super().query_data_from_date_range(
            from_timestamp=from_timestamp,
            to_timestamp=to_timestamp,
            entity=entity,
            aditional_filters=asset_filter
        )

        return super().map_data(
            response_data=response_data,
            config=config
        )

    def get_all_users(self):
        """
        Returns all the users of the protocol
        """

        config = super().get_protocol_config('user')

        response_data = super().query_data_parameter(
            entity='user'
        )

        return super().map_data(
            response_data=response_data,
            config=config
        )

    def get_all_tokens(self):
        """
        Returns all the tokens of the protocol
        """

        config = super().get_protocol_config('token')

        response_data = super().query_data_parameter(
            entity='token'
        )

        return super().map_data(
            response_data=response_data,
            config=config
        )

    def get_user_positions(self, user):
        """
        Returns the user positions (portfolio) of the given user
        """

        config = super().get_protocol_config('user_position')
        user_name = self.mappings_file['entities']['user_position']['query']['params']['user']

        response_data = super().query_data_filtered(
            entity='user_position',
            filters={user_name: user}
        )

        return super().map_data(
            response_data=response_data,
            config=config
        )

    def supported_entities(self):
        """
        Returns the supported entities for the protocol
        """
        return super().supported_entities(self.protocol_type)

    def __get_data_from_blocks__(self, from_timestamp, to_timestamp, entity, aditional_filters):
        """
        Returns the data in the specific range querying block by block
        """
        blocks = Block(protocol='block', chain=self.chain,
                       version=1)

        block_start = int(blocks.get_block_at_timestamp(from_timestamp)[
            0]['number'])
        block_end = int(blocks.get_block_at_timestamp(
            to_timestamp)[0]['number'])

        data = []
        while(block_start < block_end):
            respose = super().query_first_element(entity=entity,
                                                  timestamp=from_timestamp,
                                                  aditional_filters=aditional_filters,
                                                  block=block_end)

            data = [*data,  *respose]
            if(len(respose) == 0 or int(respose[0]['blockTimestamp']) == 0):
                block_end = block_start
            else:
                updated_timestamp = respose[0]['blockTimestamp']
                block_end = int(blocks.get_block_at_timestamp(updated_timestamp)[
                    0]['number']) - 1

        return data

    def __get_aditional_filters__(self, user, entity):
        user_filter = {}

        if(user != ''):
            user_filter = {
                self.mappings_file['entities'][entity]['query']['params']['user']: user
            }

        type = {}
        if('type' in self.mappings_file['entities'][entity]['query']['params']):
            type = {
                'type': self.mappings_file['entities'][entity]['query']['params']['type']
            }

        if (len(user_filter.items()) > 0 or len(type.items()) > 0):
            return {**user_filter, **type}

        return ''



from deficrawler.utils import get_attributes, get_filters, filter_method

import requests
import json
from retry import retry


def get_data_from(query_input, entity, from_timestamp, to_timestamp, mappings_file, endpoint, aditional_filters=""):
    """
    Gets all the existing data from the subgraph at the given time range.
    One or mor filters can be passed as parameters and will be applied in the where clause
    """
    are_data = True
    json_records = []
    iteration_timestamp = from_timestamp

    entity_name = mappings_file['entities'][entity]['query']['name']
    order_by = mappings_file['entities'][entity]['query']['params']['orderBy']
    attributes = get_attributes(entity, mappings_file)
    filters_str = get_filters(aditional_filters)

    while are_data:
        query = query_input.format(
            entity_name=entity_name,
            order_by=order_by,
            from_timestamp=iteration_timestamp,
            to_timestamp=to_timestamp,
            attributes=attributes,
            aditional_filters=filters_str
        )

        json_data = call_api(endpoint=endpoint, query=query)

        response_lenght = len(json_data['data'][entity_name])
        if (response_lenght > 0):
            list_data = json_data['data'][entity_name]
            iteration_timestamp = json_data['data'][entity_name][response_lenght - 1][order_by]

            json_records = [*json_records, *list_data]
        else:
            are_data = False

    return json_records


def get_data_parameter(query_input, entity, mappings_file, endpoint):
    """
    Gets all the existing data for the given entity.
    If this entity has some filter, in the config file, the query will apply
    this filter
    """
    are_data = True
    json_records = []

    entity_name = mappings_file['entities'][entity]['query']['name']
    order_by = mappings_file['entities'][entity]['query']['params']['orderBy']
    filter_value = mappings_file['entities'][entity]['query']['params']['initial_value']
    attributes = get_attributes(entity, mappings_file)

    while are_data:
        query = query_input.format(
            entity_name=entity_name,
            order_by=order_by,
            filter_value=filter_value,
            attributes=attributes
        )

        json_data = call_api(endpoint=endpoint, query=query)

        response_lenght = len(json_data['data'][entity_name])
        if (response_lenght > 0):
            list_data = json_data['data'][entity_name]
            filter_value = json_data['data'][entity_name][response_lenght - 1][order_by]

            json_records = [*json_records, *list_data]
        else:
            are_data = False

    return json_records


def get_data_filtered(query_input, entity, mappings_file, endpoint, filters):
    """
    Gets all the existing data from the subgraph applying the given filters
    """
    entity_name = mappings_file['entities'][entity]['query']['name']
    filters_str = get_filters(filters)

    attributes = get_attributes(entity, mappings_file)

    query = query_input.format(
        entity_name=entity_name,
        filters=filters_str,
        attributes=attributes
    )

    json_data = call_api(endpoint=endpoint, query=query)
    json_records = []

    response_lenght = len(json_data['data'][entity_name])
    if (response_lenght > 0):
        list_data = json_data['data'][entity_name]

        json_records = [*list_data]

    return json_records


def get_first_element(query_input, entity, mappings_file, endpoint, timestamp, aditional_filters, block=None):
    """
    Gets first existing data from the subgraph applying the given filters
    """
    entity_name = mappings_file['entities'][entity]['query']['name']
    filters_str = get_filters(aditional_filters)
    order_by = mappings_file['entities'][entity]['query']['params']['orderBy']
    attributes = get_attributes(entity, mappings_file)
    params = filter_method(block=block,
                           order_by_filter=order_by,
                           lte='_lte:',
                           timestamp=timestamp)

    query = query_input.format(
        entity_name=entity_name,
        order_by=order_by,
        order_by_filter=params['order_by_filter'],
        aditional_filters=filters_str,
        attributes=attributes,
        timestamp=params['timestamp'],
        lte=params['lte'],
        block=params['block']
    )

    json_data = call_api(endpoint=endpoint, query=query)
    json_records = []

    response_lenght = len(json_data['data'][entity_name])
    if (response_lenght > 0):
        list_data = json_data['data'][entity_name]

        json_records = [*list_data]

    return json_records

@retry(Exception, tries=3, delay=2)
def call_api(endpoint, query):
    """
    Returns the data from the api, if there is any error throws
    """

    response = requests.post(endpoint, json={'query': query})
    json_data = json.loads(response.text)
    if 'errors' in json_data:
        raise Exception(
            'There was an error getting the data from TheGraph' + json.dumps(json_data))

    return json_data



from deficrawler.block import Block


def test_block_bsc():
    block = Block(protocol='block', chain='bsc', version='1')
    block_info = block.get_block_at_timestamp(timestamp=1623673813)
    assert(int(block_info[0]['number']) == 8289859)

    block_info = block.get_block_info(block_number=8289859)
    assert(int(block_info[0]['timestamp']) == 1623673811)



def test_block_ethereum():
    block = Block(protocol='block', chain='Ethereum', version='1')
    block_info = block.get_block_at_timestamp(timestamp=1623673813)
    assert(int(block_info[0]['number']) == 12632560)

    block_info = block.get_block_info(block_number=12632560)
    assert(int(block_info[0]['timestamp']) == 1623673813)


def test_block_polygon():
    block = Block(protocol='block', chain='Polygon', version='1')
    block_info = block.get_block_at_timestamp(timestamp=1617176896)
    print(block_info[0]['number'])
    assert(int(block_info[0]['number']) == 12685220)

    block_info = block.get_block_info(block_number=12685220)
    assert(int(block_info[0]['timestamp']) == 1617176896)






from codecs import open
import os
from setuptools import setup, find_packages

here = os.path.abspath(os.path.dirname(__file__))

with open(os.path.join(here, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='deficrawler',
    description='Python package to query DeFi data from several The Graph subgraphs',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Apache Software License',
        'Natural Language :: English',
        'Programming Language :: Python :: 3.7',
    ],
    long_description=long_description,
    long_description_content_type='text/markdown',
    include_package_data=True,
    version='0.3.81',
    author='keyko.io',
    author_email='root@keyko.io',
    url='https://github.com/keyko-io/defi-crawler-py',
    license='MIT',
    install_requires=[
        'dict_digger',
        'requests',
        'retry'
    ],
    setup_requires=['pytest-runner'],
    tests_require=['pytest'],
    keywords=[
        'finance',
        'analysis',
    ],
    packages=find_packages(exclude=['contrib', 'docs', 'tests*']),
    python_requires='>=3'
)



from deficrawler.lending import Lending


def test_repay_aave_2_eth():
    aave = Lending(protocol="Aave", chain="Ethereum", version=2)
    repays = aave.get_data_from_date_range(
        '11/05/2021 00:00:01', '11/05/2021 00:31:00', "repay")
    assert(repays[0]['tx_id'] != "")
    assert(repays[0]['protocol'] == "Aave")
    assert(repays[0]['chain'] == "Ethereum")
    assert(repays[0]['version'] == 2)
    assert(repays[0]['user'] != "")
    assert(repays[0]['token'] != "")
    assert(repays[0]['amount'] > 0)
    assert(repays[0]['timestamp'] > 0)

def test_repay_aave_2_eth_user():
    aave = Lending(protocol="Aave", chain="Ethereum", version=2)
    repays = aave.get_data_from_date_range(
        '11/05/2021 00:00:01', '11/05/2021 00:01:00', "repay", "0xe1d18ae098ffb1ad301e0609180f155b329a710a")

    for repay in repays:
        assert(repay['user'] ==
               "0xe1d18ae098ffb1ad301e0609180f155b329a710a")


def test_repay_aave_2_polygon():
    aave = Lending(protocol="Aave", chain="Polygon", version=2)
    repays = aave.get_data_from_date_range(
        '11/05/2021 00:00:01', '11/05/2021 00:01:10', "repay")

    assert(repays[0]['tx_id'] != "")
    assert(repays[0]['protocol'] == "Aave")
    assert(repays[0]['chain'] == "Polygon")
    assert(repays[0]['version'] == 2)
    assert(repays[0]['user'] != "")
    assert(repays[0]['token'] != "")
    assert(repays[0]['amount'] > 0)
    assert(repays[0]['timestamp'] > 0)


def test_repay_aave_2_avalanche():
    aave = Lending(protocol="Aave", chain="Avalanche", version=2)
    repays = aave.get_data_from_date_range(
        '17/10/2021 00:00:01', '18/10/2021 00:01:10', "repay")

    assert(repays[0]['tx_id'] != "")
    assert(repays[0]['protocol'] == "Aave")
    assert(repays[0]['chain'] == "Avalanche")
    assert(repays[0]['version'] == 2)
    assert(repays[0]['user'] != "")
    assert(repays[0]['token'] != "")
    assert(repays[0]['amount'] > 0)
    assert(repays[0]['timestamp'] > 0)


def test_repay_aave_2_polygon_user():
    aave = Lending(protocol="Aave", chain="Polygon", version=2)
    repays = aave.get_data_from_date_range(
        '11/05/2021 00:00:01', '11/05/2021 00:01:10', "repay", "0x88b71294c66cbcb077a4626921fd8b8df6ecf042")

    for repay in repays:
        assert(repay['user'] ==
               "0x88b71294c66cbcb077a4626921fd8b8df6ecf042")


def test_repay_compound_2_eth():
    compound = Lending(protocol="Compound", chain="Ethereum", version=2)
    repays = compound.get_data_from_date_range(
        '11/05/2021 00:00:01', '11/05/2021 00:31:10', "repay")

    assert(repays[0]['tx_id'] != "")
    assert(repays[0]['protocol'] == "Compound")
    assert(repays[0]['chain'] == "Ethereum")
    assert(repays[0]['version'] == 2)
    assert(repays[0]['user'] != "")
    assert(repays[0]['token'] != "")
    assert(float(repays[0]['amount']) > 0)
    assert(repays[0]['timestamp'] > 0)

def test_repay_compound_2_eth_user():
    compound = Lending(protocol="Compound", chain="Ethereum", version=2)
    repays = compound.get_data_from_date_range(
        '11/05/2021 00:00:01', '11/05/2021 00:01:10', "repay", "0x5fec8b5630fb7b923d9308902856054996829f60")

    for repay in repays:
        assert(repay['user'] ==
               "0x5fec8b5630fb7b923d9308902856054996829f60")

def test_repay_cream_2_eth():
    cream = Lending(protocol="Cream", chain="Ethereum", version=2)
    repays = cream.get_data_from_date_range(
        '11/05/2021 00:00:01', '12/05/2021 11:54:10', "repay")

    assert(repays[0]['tx_id'] != "")
    assert(repays[0]['protocol'] == "Cream")
    assert(repays[0]['chain'] == "Ethereum")
    assert(repays[0]['version'] == 2)
    assert(repays[0]['user'] != "")
    assert(repays[0]['token'] != "")
    assert(float(repays[0]['amount']) > 0)
    assert(repays[0]['timestamp'] > 0)


def test_repay_cream_2_arbitrum():
    cream = Lending(protocol="Cream", chain="Arbitrum", version=2)
    repays = cream.get_data_from_date_range(
        '20/09/2021 00:00:01', '26/09/2021 11:54:10', "repay")

    assert(repays[0]['tx_id'] != "")
    assert(repays[0]['protocol'] == "Cream")
    assert(repays[0]['chain'] == "Arbitrum")
    assert(repays[0]['version'] == 2)
    assert(repays[0]['user'] != "")
    assert(repays[0]['token'] != "")
    assert(float(repays[0]['amount']) > 0)
    assert(repays[0]['timestamp'] > 0)

def test_repay_cream_2_polygon():
    cream = Lending(protocol="Cream", chain="Polygon", version=2)
    repays = cream.get_data_from_date_range(
        '25/09/2021 00:00:01', '26/09/2021 11:54:10', "repay")

    assert(repays[0]['tx_id'] != "")
    assert(repays[0]['protocol'] == "Cream")
    assert(repays[0]['chain'] == "Polygon")
    assert(repays[0]['version'] == 2)
    assert(repays[0]['user'] != "")
    assert(repays[0]['token'] != "")
    assert(float(repays[0]['amount']) > 0)
    assert(repays[0]['timestamp'] > 0)

def test_repay_cream_2_eth_user():
    cream = Lending(protocol="Cream", chain="Ethereum", version=2)
    repays = cream.get_data_from_date_range(
        '11/05/2021 00:00:01', '12/05/2021 11:54:10', "repay", "0xf662eba33a8630b51a3d955213dd7a58c2a687a9")

    for repay in repays:
        assert(repay['user'] ==
               "0xf662eba33a8630b51a3d955213dd7a58c2a687a9")

def test_repay_cream_2_bsc():
    cream = Lending(protocol="Cream", chain="bsc", version=2)
    repays = cream.get_data_from_date_range(
        '08/05/2021 00:00:01', '12/05/2021 11:54:10', "repay")

    assert(repays[0]['tx_id'] != "")
    assert(repays[0]['protocol'] == "Cream")
    assert(repays[0]['chain'] == "bsc")
    assert(repays[0]['version'] == 2)
    assert(repays[0]['user'] != "")
    assert(repays[0]['token'] != "")
    assert(float(repays[0]['amount']) > 0)
    assert(repays[0]['timestamp'] > 0)



def test_repay_cream_2_bsc():
    cream = Lending(protocol="Cream", chain="avalanche", version=2)
    repays = cream.get_data_from_date_range(
        '18/10/2021 00:00:01', '19/10/2021 00:00:10', "repay")

    assert(repays[0]['tx_id'] != "")
    assert(repays[0]['protocol'] == "Cream")
    assert(repays[0]['chain'] == "avalanche")
    assert(repays[0]['version'] == 2)
    assert(repays[0]['user'] != "")
    assert(repays[0]['token'] != "")
    assert(float(repays[0]['amount']) > 0)
    assert(repays[0]['timestamp'] > 0)


def test_repay_cream_2_bsc():
    cream = Lending(protocol="Cream", chain="bsc", version=2)
    repays = cream.get_data_from_date_range(
        '08/05/2021 00:00:01', '12/05/2021 11:54:10', "repay", "0x86caaf4e8592cbe7f93a8ccec1c4be6b61be7693")

    for repay in repays:
        assert(repay['user'] ==
               "0x86caaf4e8592cbe7f93a8ccec1c4be6b61be7693")


def test_repay_kashi_1_eth():
    kashi = Lending(protocol="Kashi", chain="Ethereum", version=1)
    repays = kashi.get_data_from_date_range(
        '20/09/2021 00:00:01', '20/09/2021 11:54:10', "repay")

    assert(repays[0]['tx_id'] != "")
    assert(repays[0]['protocol'] == "Kashi")
    assert(repays[0]['chain'] == "Ethereum")
    assert(repays[0]['version'] == 1)
    assert(repays[0]['user'] != "")
    assert(repays[0]['token'] != "")
    assert(float(repays[0]['amount']) > 0)
    assert(repays[0]['timestamp'] > 0)


def test_repay_kashi_1_polygon():
    kashi = Lending(protocol="Kashi", chain="Polygon", version=1)
    repays = kashi.get_data_from_date_range(
        '20/09/2021 00:00:01', '20/09/2021 11:54:10', "repay")

    assert(repays[0]['tx_id'] != "")
    assert(repays[0]['protocol'] == "Kashi")
    assert(repays[0]['chain'] == "Polygon")
    assert(repays[0]['version'] == 1)
    assert(repays[0]['user'] != "")
    assert(repays[0]['token'] != "")
    assert(float(repays[0]['amount']) > 0)
    assert(repays[0]['timestamp'] > 0)



from deficrawler.protocol_base import ProtocolBase

import pytest


def test_protocol_not_exits():
    with pytest.raises(Exception) as execinfo:
        ProtocolBase(protocol="NA", version=12, chain="other")

    assert execinfo.value.args[0] == 'Protocol NA version 12 not supported'


def test_enpoint_not_exits():
    with pytest.raises(Exception) as execinfo:
        ProtocolBase(protocol="Aave", version=2, chain="other")

    assert execinfo.value.args[0] == 'Protocol Aave chain other not supported'


def test_entity_not_exits():
    with pytest.raises(Exception) as execinfo:
        protocol = ProtocolBase(protocol="Compound", version=2, chain="Ethereum")
        protocol.get_protocol_config('swap')

    assert execinfo.value.args[0] == 'Entity swap not supported for protocol Compound'
    

def test_protocol_exits():
    protocol = ProtocolBase(protocol="Compound", version=2, chain="Ethereum")
    config = protocol.get_protocol_config('borrow')
    assert('attributes' in config)
    assert('transformations' in config)
    assert('query_elements' in config)








from deficrawler.dex import Dex


def test_swap_uniswap_2_eth():
    uniswap = Dex(protocol="Uniswap", chain="Ethereum", version=2)
    swaps = uniswap.get_data_from_date_range(
        '04/05/2021 00:00:00', '04/05/2021 00:00:30', "swap")
    assert(swaps[0]['tx_id'] != "")
    assert(swaps[0]['protocol'] == "Uniswap")
    assert(swaps[0]['chain'] == "Ethereum")
    assert(swaps[0]['version'] == 2)
    assert(swaps[0]['user'] != "")
    assert(swaps[0]['from_token'] != "")
    assert(swaps[0]['to_token'] != "")
    assert(swaps[0]['pool'] != "")
    assert(swaps[0]['pool'] != "")
    assert(float(swaps[0]['amount_usd']) > 0)
    assert(float(swaps[0]['from_token_amount']) > 0)
    assert(float(swaps[0]['to_token_amount']) > 0)
    assert(float(swaps[0]['timestamp']) > 0)


def test_swap_balancer_1_eth():
    uniswap = Dex(protocol="Balancer", chain="Ethereum", version=1)
    swaps = uniswap.get_data_from_date_range(
        '04/05/2021 00:00:00', '04/05/2021 00:01:30', "swap")
    assert(swaps[0]['tx_id'] != "")
    assert(swaps[0]['protocol'] == "Balancer")
    assert(swaps[0]['chain'] == "Ethereum")
    assert(swaps[0]['version'] == 1)
    assert(swaps[0]['user'] != "")
    assert(swaps[0]['from_token'] != "")
    assert(swaps[0]['to_token'] != "")
    assert(swaps[0]['pool'] != "")
    assert(float(swaps[0]['from_token_amount']) > 0)
    assert(float(swaps[0]['to_token_amount']) > 0)
    assert(float(swaps[0]['timestamp']) > 0)


def test_swap_bancor_1_eth():
    uniswap = Dex(protocol="Bancor", chain="Ethereum", version=1)
    swaps = uniswap.get_data_from_date_range(
        '04/05/2021 00:00:00', '04/05/2021 04:00:30', "swap")
    assert(swaps[0]['tx_id'] != "")
    assert(swaps[0]['protocol'] == "Bancor")
    assert(swaps[0]['chain'] == "Ethereum")
    assert(swaps[0]['version'] == 1)
    assert(swaps[0]['user'] != "")
    assert(swaps[0]['from_token'] != "")
    assert(swaps[0]['to_token'] != "")
    assert(float(swaps[0]['from_token_amount']) > 0)
    assert(float(swaps[0]['to_token_amount']) > 0)
    assert(float(swaps[0]['timestamp']) > 0)


def test_swap_shushi_1_eth():
    shushi = Dex(protocol="SushiSwap", chain="Ethereum", version=1)
    swaps = shushi.get_data_from_date_range(
        '04/05/2021 00:00:00', '04/05/2021 0:31:30', "swap")
    assert(swaps[0]['tx_id'] != "")
    assert(swaps[0]['protocol'] == "SushiSwap")
    assert(swaps[0]['chain'] == "Ethereum")
    assert(swaps[0]['version'] == 1)
    assert(swaps[0]['user'] != "")
    assert(swaps[0]['from_token'] != "")
    assert(swaps[0]['to_token'] != "")
    assert(swaps[0]['pool'] != "")
    assert(float(swaps[0]['amount_usd']) > 0)
    assert(float(swaps[0]['from_token_amount']) > 0)
    assert(float(swaps[0]['to_token_amount']) > 0)
    assert(float(swaps[0]['timestamp']) > 0)

    list_swaps = shushi.get_data_from_date_range(
        '04/05/2021 00:00:00', '04/05/2021 9:01:30', "swap", pool='0xceff51756c56ceffca006cd410b03ffc46dd3a58'
    )

    assert(len(list_swaps) > 0)


def test_swap_shushi_1_bsc():
    shushi = Dex(protocol="SushiSwap", chain="bsc", version=1)
    swaps = shushi.get_data_from_date_range(
        '05/05/2021 00:00:00', '05/05/2021 0:31:30', "swap")
    assert(swaps[0]['tx_id'] != "")
    assert(swaps[0]['protocol'] == "SushiSwap")
    assert(swaps[0]['chain'] == "bsc")
    assert(swaps[0]['version'] == 1)
    assert(swaps[0]['user'] != "")
    assert(swaps[0]['from_token'] != "")
    assert(swaps[0]['to_token'] != "")
    assert(swaps[0]['pool'] != "")
    assert(float(swaps[0]['amount_usd']) >= 0)
    assert(float(swaps[0]['from_token_amount']) > 0)
    assert(float(swaps[0]['to_token_amount']) > 0)
    assert(float(swaps[0]['timestamp']) > 0)


def test_swap_shushi_1_polygon():
    shushi = Dex(protocol="SushiSwap", chain="polygon", version=1)
    swaps = shushi.get_data_from_date_range(
        '04/05/2021 00:00:00', '04/05/2021 0:01:30', "swap")
    assert(swaps[0]['tx_id'] != "")
    assert(swaps[0]['protocol'] == "SushiSwap")
    assert(swaps[0]['chain'] == "polygon")
    assert(swaps[0]['version'] == 1)
    assert(swaps[0]['user'] != "")
    assert(swaps[0]['from_token'] != "")
    assert(swaps[0]['to_token'] != "")
    assert(swaps[0]['pool'] != "")
    assert(float(swaps[0]['amount_usd']) > 0)
    assert(float(swaps[0]['from_token_amount']) > 0)
    assert(float(swaps[0]['to_token_amount']) > 0)
    assert(float(swaps[0]['timestamp']) > 0)


def test_swap_shushi_1_celo():
    shushi = Dex(protocol="SushiSwap", chain="celo", version=1)
    swaps = shushi.get_data_from_date_range(
        '17/10/2021 00:00:00', '17/10/2021 3:01:30', "swap")
    assert(swaps[0]['tx_id'] != "")
    assert(swaps[0]['protocol'] == "SushiSwap")
    assert(swaps[0]['chain'] == "celo")
    assert(swaps[0]['version'] == 1)
    assert(swaps[0]['user'] != "")
    assert(swaps[0]['from_token'] != "")
    assert(swaps[0]['to_token'] != "")
    assert(swaps[0]['pool'] != "")
    assert(float(swaps[0]['amount_usd']) > 0)
    assert(float(swaps[0]['from_token_amount']) > 0)
    assert(float(swaps[0]['to_token_amount']) > 0)
    assert(float(swaps[0]['timestamp']) > 0)


def test_swap_shushi_1_fantom():
    shushi = Dex(protocol="SushiSwap", chain="fantom", version=1)
    swaps = shushi.get_data_from_date_range(
        '31/08/2021 00:00:00', '31/08/2021 04:00:00', "swap")
    assert(swaps[0]['tx_id'] != "")
    assert(swaps[0]['protocol'] == "SushiSwap")
    assert(swaps[0]['chain'] == "fantom")
    assert(swaps[0]['version'] == 1)
    assert(swaps[0]['user'] != "")
    assert(swaps[0]['from_token'] != "")
    assert(swaps[0]['to_token'] != "")
    assert(swaps[0]['pool'] != "")
    assert(float(swaps[0]['amount_usd']) > 0)
    assert(float(swaps[0]['from_token_amount']) > 0)
    assert(float(swaps[0]['to_token_amount']) > 0)
    assert(float(swaps[0]['timestamp']) > 0)


def test_swap_uniswap_3_eth():
    uniswap = Dex(protocol="Uniswap", chain="Ethereum", version=3)
    swaps = uniswap.get_data_from_date_range(
        '31/08/2021 00:00:00', '31/08/2021 00:10:00', "swap")
    assert(swaps[0]['tx_id'] != "")
    assert(swaps[0]['protocol'] == "Uniswap")
    assert(swaps[0]['chain'] == "Ethereum")
    assert(swaps[0]['version'] == 3)
    assert(swaps[0]['user'] != "")
    assert(swaps[0]['from_token'] != "")
    assert(swaps[0]['to_token'] != "")
    assert(swaps[0]['pool'] != "")
    assert(float(swaps[0]['amount_usd']) > 0)
    assert(float(swaps[0]['from_token_amount']) > 0)
    assert(float(swaps[0]['to_token_amount']) > 0)
    assert(float(swaps[0]['timestamp']) > 0)


def test_swap_uniswap_3_optimism():
    uniswap = Dex(protocol="Uniswap", chain="optimism", version=3)
    swaps = uniswap.get_data_from_date_range(
        '18/10/2021 00:00:00', '18/10/2021 02:10:00', "swap")
    assert(swaps[0]['tx_id'] != "")
    assert(swaps[0]['protocol'] == "Uniswap")
    assert(swaps[0]['chain'] == "optimism")
    assert(swaps[0]['version'] == 3)
    assert(swaps[0]['user'] != "")
    assert(swaps[0]['from_token'] != "")
    assert(swaps[0]['to_token'] != "")
    assert(swaps[0]['pool'] != "")
    assert(float(swaps[0]['amount_usd']) > 0)
    assert(float(swaps[0]['from_token_amount']) > 0)
    assert(float(swaps[0]['to_token_amount']) > 0)
    assert(float(swaps[0]['timestamp']) > 0)


# def test_swap_ubeswap_1_celo():
#     ubeswap = Dex(protocol="ubeswap", chain="celo", version=1)
#     swaps = ubeswap.get_data_from_date_range(
#         '04/05/2021 00:00:00', '04/05/2021 0:01:30', "swap")

#     assert(swaps[0]['tx_id'] != "")
#     assert(swaps[0]['protocol'] == "ubeswap")
#     assert(swaps[0]['chain'] == "celo")
#     assert(swaps[0]['version'] == 1)
#     assert(swaps[0]['user'] != "")
#     assert(swaps[0]['from_token'] != "")
#     assert(swaps[0]['to_token'] != "")
#     assert(swaps[0]['pool'] != "")
#     assert(float(swaps[0]['from_token_amount']) > 0)
#     assert(float(swaps[0]['to_token_amount']) > 0)
#     assert(float(swaps[0]['timestamp']) > 0)


def test_swap_balancer_2_eth():
    balancer = Dex(protocol="balancer", chain="ethereum", version=2)
    swaps = balancer.get_data_from_date_range(
        '31/08/2021 00:00:00', '31/08/2021 01:00:00', "swap")

    assert(swaps[0]['tx_id'] != "")
    assert(swaps[0]['protocol'] == "balancer")
    assert(swaps[0]['chain'] == "ethereum")
    assert(swaps[0]['version'] == 2)
    assert(swaps[0]['user'] != "")
    assert(swaps[0]['from_token'] != "")
    assert(swaps[0]['to_token'] != "")
    assert(swaps[0]['pool'] != "")
    assert(swaps[0]['amount_usd'] == 'NA')
    assert(float(swaps[0]['from_token_amount']) > 0)
    assert(float(swaps[0]['to_token_amount']) > 0)
    assert(float(swaps[0]['timestamp']) > 0)


def test_swap_balancer_2_polygon():
    balancer = Dex(protocol="balancer", chain="polygon", version=2)
    swaps = balancer.get_data_from_date_range(
        '31/08/2021 00:00:00', '31/08/2021 01:00:00', "swap")

    assert(swaps[0]['tx_id'] != "")
    assert(swaps[0]['protocol'] == "balancer")
    assert(swaps[0]['chain'] == "polygon")
    assert(swaps[0]['version'] == 2)
    assert(swaps[0]['user'] != "")
    assert(swaps[0]['from_token'] != "")
    assert(swaps[0]['to_token'] != "")
    assert(swaps[0]['pool'] != "")
    assert(swaps[0]['amount_usd'] == 'NA')
    assert(float(swaps[0]['from_token_amount']) > 0)
    assert(float(swaps[0]['to_token_amount']) > 0)
    assert(float(swaps[0]['timestamp']) > 0)


def test_swap_balancer_2_arbitrum():
    balancer = Dex(protocol="balancer", chain="arbitrum", version=2)
    swaps = balancer.get_data_from_date_range(
        '31/08/2021 00:00:00', '01/09/2021 0:01:30', "swap")

    assert(swaps[0]['tx_id'] != "")
    assert(swaps[0]['protocol'] == "balancer")
    assert(swaps[0]['chain'] == "arbitrum")
    assert(swaps[0]['version'] == 2)
    assert(swaps[0]['user'] != "")
    assert(swaps[0]['from_token'] != "")
    assert(swaps[0]['to_token'] != "")
    assert(swaps[0]['pool'] != "")
    assert(swaps[0]['amount_usd'] == 'NA')
    assert(float(swaps[0]['from_token_amount']) > 0)
    assert(float(swaps[0]['to_token_amount']) > 0)
    assert(float(swaps[0]['timestamp']) > 0)


def test_swap_pancakeswap_2_bsc():
    pancakeswap = Dex(protocol="pancakeswap", chain="bsc", version=2)
    swaps = pancakeswap.get_data_from_date_range(
        '31/08/2021 00:00:00', '01/09/2021 0:01:30', "swap")

    assert(swaps[0]['tx_id'] != "")
    assert(swaps[0]['protocol'] == "pancakeswap")
    assert(swaps[0]['chain'] == "bsc")
    assert(swaps[0]['version'] == 2)
    assert(swaps[0]['user'] != "")
    assert(swaps[0]['from_token'] != "")
    assert(swaps[0]['to_token'] != "")
    assert(swaps[0]['pool'] != "")
    assert(float(swaps[0]['amount_usd']) >= 0)
    assert(float(swaps[0]['from_token_amount']) > 0)
    assert(float(swaps[0]['to_token_amount']) > 0)
    assert(float(swaps[0]['timestamp']) > 0)


def test_swap_quickswap_1_polygon():
    quickswap = Dex(protocol="quickswap", chain="polygon", version=1)
    swaps = quickswap.get_data_from_date_range(
        '31/08/2021 00:00:00', '31/08/2021 0:01:30', "swap")

    assert(swaps[0]['tx_id'] != "")
    assert(swaps[0]['protocol'] == "quickswap")
    assert(swaps[0]['chain'] == "polygon")
    assert(swaps[0]['version'] == 1)
    assert(swaps[0]['user'] != "")
    assert(swaps[0]['from_token'] != "")
    assert(swaps[0]['to_token'] != "")
    assert(swaps[0]['pool'] != "")
    assert(float(swaps[0]['amount_usd']) > 0)
    assert(float(swaps[0]['from_token_amount']) > 0)
    assert(float(swaps[0]['to_token_amount']) > 0)
    assert(float(swaps[0]['timestamp']) > 0)


def test_swap_pangolin_1_avalanche():
    pangolin = Dex(protocol="pangolin", chain="avalanche", version=1)
    swaps = pangolin.get_data_from_date_range(
        '31/08/2021 00:00:00', '31/08/2021 0:01:30', "swap")

    assert(swaps[0]['tx_id'] != "")
    assert(swaps[0]['protocol'] == "pangolin")
    assert(swaps[0]['chain'] == "avalanche")
    assert(swaps[0]['version'] == 1)
    assert(swaps[0]['user'] != "")
    assert(swaps[0]['from_token'] != "")
    assert(swaps[0]['to_token'] != "")
    assert(swaps[0]['pool'] != "")
    assert(float(swaps[0]['amount_usd']) > 0)
    assert(float(swaps[0]['from_token_amount']) > 0)
    assert(float(swaps[0]['to_token_amount']) > 0)
    assert(float(swaps[0]['timestamp']) > 0)


def test_swap_traderjoe_1_avalanche():
    traderjoe = Dex(protocol="traderjoe", chain="avalanche", version=1)
    swaps = traderjoe.get_data_from_date_range(
        '31/08/2021 00:00:00', '31/08/2021 0:01:30', "swap")

    assert(swaps[0]['tx_id'] != "")
    assert(swaps[0]['protocol'] == "traderjoe")
    assert(swaps[0]['chain'] == "avalanche")
    assert(swaps[0]['version'] == 1)
    assert(swaps[0]['user'] != "")
    assert(swaps[0]['from_token'] != "")
    assert(swaps[0]['to_token'] != "")
    assert(swaps[0]['pool'] != "")
    assert(float(swaps[0]['amount_usd']) > 0)
    assert(float(swaps[0]['from_token_amount']) > 0)
    assert(float(swaps[0]['to_token_amount']) > 0)
    assert(float(swaps[0]['timestamp']) > 0)


def test_swap_dodoex_2_ethereum():
    dodoex = Dex(protocol="dodoex", chain="ethereum", version=2)
    swaps = dodoex.get_data_from_date_range(
        '17/10/2021 00:00:00', '17/10/2021 1:00:30', "swap")

    assert(swaps[0]['tx_id'] != "")
    assert(swaps[0]['protocol'] == "dodoex")
    assert(swaps[0]['chain'] == "ethereum")
    assert(swaps[0]['version'] == 2)
    assert(swaps[0]['user'] != "")
    assert(swaps[0]['from_token'] != "")
    assert(swaps[0]['to_token'] != "")
    assert(swaps[0]['pool'] != "")
    assert(float(swaps[0]['amount_usd']) > 0)
    assert(float(swaps[0]['from_token_amount']) > 0)
    assert(float(swaps[0]['to_token_amount']) > 0)
    assert(float(swaps[0]['timestamp']) > 0)


def test_swap_dodoex_2_polygon():
    dodoex = Dex(protocol="dodoex", chain="polygon", version=2)
    swaps = dodoex.get_data_from_date_range(
        '17/10/2021 00:00:00', '17/10/2021 1:00:30', "swap")

    assert(swaps[0]['tx_id'] != "")
    assert(swaps[0]['protocol'] == "dodoex")
    assert(swaps[0]['chain'] == "polygon")
    assert(swaps[0]['version'] == 2)
    assert(swaps[0]['user'] != "")
    assert(swaps[0]['from_token'] != "")
    assert(swaps[0]['to_token'] != "")
    assert(swaps[0]['pool'] != "")
    assert(float(swaps[0]['amount_usd']) > 0)
    assert(float(swaps[0]['from_token_amount']) > 0)
    assert(float(swaps[0]['to_token_amount']) > 0)
    assert(float(swaps[0]['timestamp']) > 0)


def test_swap_dodoex_2_bsc():
    dodoex = Dex(protocol="dodoex", chain="bsc", version=2)
    swaps = dodoex.get_data_from_date_range(
        '17/10/2021 00:00:00', '17/10/2021 1:00:30', "swap")

    assert(swaps[0]['tx_id'] != "")
    assert(swaps[0]['protocol'] == "dodoex")
    assert(swaps[0]['chain'] == "bsc")
    assert(swaps[0]['version'] == 2)
    assert(swaps[0]['user'] != "")
    assert(swaps[0]['from_token'] != "")
    assert(swaps[0]['to_token'] != "")
    assert(swaps[0]['pool'] != "")
    assert(float(swaps[0]['amount_usd']) > 0)
    assert(float(swaps[0]['from_token_amount']) > 0)
    assert(float(swaps[0]['to_token_amount']) > 0)
    assert(float(swaps[0]['timestamp']) > 0)


def test_swap_dodoex_2_arbitrum():
    dodoex = Dex(protocol="dodoex", chain="arbitrum", version=2)
    swaps = dodoex.get_data_from_date_range(
        '17/10/2021 00:00:00', '17/10/2021 1:00:30', "swap")

    assert(swaps[0]['tx_id'] != "")
    assert(swaps[0]['protocol'] == "dodoex")
    assert(swaps[0]['chain'] == "arbitrum")
    assert(swaps[0]['version'] == 2)
    assert(swaps[0]['user'] != "")
    assert(swaps[0]['from_token'] != "")
    assert(swaps[0]['to_token'] != "")
    assert(swaps[0]['pool'] != "")
    assert(float(swaps[0]['amount_usd']) >= 0)
    assert(float(swaps[0]['from_token_amount']) > 0)
    assert(float(swaps[0]['to_token_amount']) > 0)
    assert(float(swaps[0]['timestamp']) > 0)


from deficrawler.transformer import Transformer


def test_no_transformation():
    element = {'id': '0xd522f7c5f29bb6a38d7bbeb08938afb619c8e8a7318a0af0815d4a9d4a8330ac-94',
               'borrower': '0x3f3e305c4ad49271ebda489dd43d2c8f027d2d41',
               'underlyingSymbol': 'USDC', 'amount': '500000', 'blockTime': 1618956398}
    common_field = "user"
    protocol_field = ['borrower']
    transformations = {'tx_id': 'tx_id_hyphen'}
    query_elements = {'tx_id': ['id'], 'user': ['borrower'],
                      'token': ['underlyingSymbol'], 'amount': ['amount'],
                      'timestamp': ['blockTime']}

    transformer = Transformer()

    borrower = transformer.transform(
        element,
        common_field,
        protocol_field,
        transformations,
        query_elements
    )

    assert(borrower == '0x3f3e305c4ad49271ebda489dd43d2c8f027d2d41')


def test_decimals():
    element = {'amount': '40000000000', 'id': '0x0fcfca3f64964cd19accad44dbd9e51735cea022224d6cc8aeeed5b418a55f0b:2',
               'reserve': {'decimals': 6, 'symbol': 'USDT'}, 'timestamp': 1618956015,
               'user': {'id': '0x827e2244086565cf8fd7039eb12eb855f56ef167'}}
    common_field = "amount"
    protocol_field = ['amount']
    transformations = {'amount': 'decimals', 'tx_id': 'tx_id_colon'}
    query_elements = {'decimals': ['reserve', 'decimals'], 'tx_id': ['id'], 'user': ['user', 'id'],
                      'token': ['reserve', 'symbol'], 'amount': ['amount'], 'timestamp': ['timestamp']}

    transformer = Transformer()

    decimals = transformer.transform(
        element,
        common_field,
        protocol_field,
        transformations,
        query_elements
    )

    assert(decimals == 40000000000 / 10 ** 6)


def test_principal_decimals():
    element = {'collateralAmount': '8390418', 'collateralReserve': {'decimals': 8, 'symbol': 'WBTC'},
               'id': '0x14e3d6aee390617fde0fb893bcc25401bb981d1321242f3010f26da88b16a099:8',
               'liquidator': '0x645e93859ec63abe0c7fe74f17c07c236ee58799', 'principalAmount':
               '1861472948139816454', 'principalReserve': {'decimals': 18, 'symbol': 'WETH'},
               'timestamp': 1618982463, 'user': {'id': '0xcfd873f19a86b84cfc4916e8623f2486dc83d792'}}

    common_field = "amount_principal"
    protocol_field = ['principalAmount']
    transformations = {'amount_principal': 'principal_decimals',
                       'amount_collateral': 'collateral_decimals', 'tx_id': 'tx_id_colon'}
    query_elements = {'principal_decimals': ['principalReserve', 'decimals'],
                      'collateral_decimals': ['collateralReserve', 'decimals'], 'tx_id': ['id'],
                      'user': ['user', 'id'], 'token_principal': ['principalReserve', 'symbol'],
                      'token_collateral': ['collateralReserve', 'symbol'], 'amount_principal': ['principalAmount'],
                      'amount_collateral': ['collateralAmount'], 'liquidator': ['liquidator'], 'timestamp': ['timestamp']}

    transformer = Transformer()

    decimals = transformer.transform(
        element,
        common_field,
        protocol_field,
        transformations,
        query_elements
    )

    assert(decimals == 1861472948139816454 / 10 ** 18)


def test_collateral_decimals():
    element = {'collateralAmount': '8390418', 'collateralReserve': {'decimals': 8, 'symbol': 'WBTC'},
               'id': '0x14e3d6aee390617fde0fb893bcc25401bb981d1321242f3010f26da88b16a099:8',
               'liquidator': '0x645e93859ec63abe0c7fe74f17c07c236ee58799', 'principalAmount':
               '1861472948139816454', 'principalReserve': {'decimals': 18, 'symbol': 'WETH'},
               'timestamp': 1618982463, 'user': {'id': '0xcfd873f19a86b84cfc4916e8623f2486dc83d792'}}

    common_field = "amount_collateral"
    protocol_field = ['collateralAmount']
    transformations = {'amount_principal': 'principal_decimals',
                       'amount_collateral': 'collateral_decimals', 'tx_id': 'tx_id_colon'}
    query_elements = {'principal_decimals': ['principalReserve', 'decimals'],
                      'collateral_decimals': ['collateralReserve', 'decimals'], 'tx_id': ['id'],
                      'user': ['user', 'id'], 'token_principal': ['principalReserve', 'symbol'],
                      'token_collateral': ['collateralReserve', 'symbol'], 'amount_principal': ['principalAmount'],
                      'amount_collateral': ['collateralAmount'], 'liquidator': ['liquidator'], 'timestamp': ['timestamp']}

    transformer = Transformer()

    decimals = transformer.transform(
        element,
        common_field,
        protocol_field,
        transformations,
        query_elements
    )

    assert(decimals == 8390418 / 10 ** 8)


def test_from_token():
    element = {'amount0In': '0.1', 'amount0Out': '0', 'amount1In': '223212862.542378007',
               'amount1Out': '777931322218.496933182', 'pair': {'token0': {'symbol': 'WETH'}, 'token1': {'symbol': 'HOKK'}},
               'sender': '0x7a250d5630b4cf539739df2c5dacb4c659f2488d',
               'timestamp': '1620079287', 'transaction': {'id': '0xf7bec9d32958801feb1fc06a7dee136e1de844ddbf801ebbb7967b0e39392333'}}

    common_field = "from_token"
    protocol_field = [['pair', 'token0', 'symbol'],
                      ['pair', 'token1', 'symbol']]
    transformations = {'from_token': 'from_token', 'to_token': 'to_token',
                       'from_token_amount': 'from_token_amount', 'to_token_amount': 'to_token_amount'}
    query_elements = {'user': ['sender'], 'tx_id': ['transaction', 'id'], 'from_token': [['pair', 'token0', 'symbol'], ['pair', 'token1', 'symbol']], 'to_token': [['pair', 'token0', 'symbol'], [
        'pair', 'token1', 'symbol']], 'from_token_amount': [['amount0In'], ['amount1In']], 'to_token_amount': [['amount0Out'], ['amount1Out']], 'timestamp': ['timestamp']}

    transformer = Transformer()

    from_token = transformer.transform(
        element,
        common_field,
        protocol_field,
        transformations,
        query_elements
    )

    assert(from_token == 'WETH')


def test_to_token():
    element = {'amount0In': '0.1', 'amount0Out': '0', 'amount1In': '223212862.542378007',
               'amount1Out': '777931322218.496933182', 'pair': {'token0': {'symbol': 'WETH'}, 'token1': {'symbol': 'HOKK'}},
               'sender': '0x7a250d5630b4cf539739df2c5dacb4c659f2488d',
               'timestamp': '1620079287', 'transaction': {'id': '0xf7bec9d32958801feb1fc06a7dee136e1de844ddbf801ebbb7967b0e39392333'}}

    common_field = "to_token"
    protocol_field = [['pair', 'token0', 'symbol'],
                      ['pair', 'token1', 'symbol']]
    transformations = {'from_token': 'from_token', 'to_token': 'to_token',
                       'from_token_amount': 'from_token_amount', 'to_token_amount': 'to_token_amount'}
    query_elements = {'user': ['sender'], 'tx_id': ['transaction', 'id'], 'from_token': [['pair', 'token0', 'symbol'], ['pair', 'token1', 'symbol']], 'to_token': [['pair', 'token0', 'symbol'], [
        'pair', 'token1', 'symbol']], 'from_token_amount': [['amount0In'], ['amount1In']], 'to_token_amount': [['amount0Out'], ['amount1Out']], 'timestamp': ['timestamp']}

    transformer = Transformer()

    to_token = transformer.transform(
        element,
        common_field,
        protocol_field,
        transformations,
        query_elements
    )

    assert(to_token == 'HOKK')


def test_from_token_amount():
    element = {'amount0In': '0.1', 'amount0Out': '0', 'amount1In': '223212862.542378007',
               'amount1Out': '777931322218.496933182', 'pair': {'token0': {'symbol': 'WETH'}, 'token1': {'symbol': 'HOKK'}},
               'sender': '0x7a250d5630b4cf539739df2c5dacb4c659f2488d',
               'timestamp': '1620079287', 'transaction': {'id': '0xf7bec9d32958801feb1fc06a7dee136e1de844ddbf801ebbb7967b0e39392333'}}

    common_field = "from_token_amount"
    protocol_field = [["amount0In"],
                      ["amount1In"]]
    transformations = {'from_token': 'from_token', 'to_token': 'to_token',
                       'from_token_amount': 'from_token_amount', 'to_token_amount': 'to_token_amount'}
    query_elements = {'user': ['sender'], 'tx_id': ['transaction', 'id'], 'from_token': [['pair', 'token0', 'symbol'], ['pair', 'token1', 'symbol']], 'to_token': [['pair', 'token0', 'symbol'], [
        'pair', 'token1', 'symbol']], 'from_token_amount': [['amount0In'], ['amount1In']], 'to_token_amount': [['amount0Out'], ['amount1Out']], 'timestamp': ['timestamp']}

    transformer = Transformer()

    amount = transformer.transform(
        element,
        common_field,
        protocol_field,
        transformations,
        query_elements
    )

    assert(amount == '0.1')


def test_to_token_amount():
    element = {'amount0In': '0.1', 'amount0Out': '0', 'amount1In': '223212862.542378007',
               'amount1Out': '777931322218.496933182', 'pair': {'token0': {'symbol': 'WETH'}, 'token1': {'symbol': 'HOKK'}},
               'sender': '0x7a250d5630b4cf539739df2c5dacb4c659f2488d',
               'timestamp': '1620079287', 'transaction': {'id': '0xf7bec9d32958801feb1fc06a7dee136e1de844ddbf801ebbb7967b0e39392333'}}

    common_field = "to_token_amount"
    protocol_field = [["amount0Out"],
                      ["amount1Out"]]
    transformations = {'from_token': 'from_token', 'to_token': 'to_token',
                       'from_token_amount': 'from_token_amount', 'to_token_amount': 'to_token_amount'}
    query_elements = {'user': ['sender'], 'tx_id': ['transaction', 'id'], 'from_token': [['pair', 'token0', 'symbol'], ['pair', 'token1', 'symbol']], 'to_token': [['pair', 'token0', 'symbol'], [
        'pair', 'token1', 'symbol']], 'from_token_amount': [['amount0In'], ['amount1In']], 'to_token_amount': [['amount0Out'], ['amount1Out']], 'timestamp': ['timestamp']}

    transformer = Transformer()

    amount = transformer.transform(
        element,
        common_field,
        protocol_field,
        transformations,
        query_elements
    )

    assert(amount == '777931322218.496933182')


def test_array_length():
    element = {'borrowedReservesCount': 3, 'id': '0x017389d8542b201e4626dde118fd359b972c654d',
               'liquidationCallHistory': [{'id': '0xa81e545027dcde712981a9e32ef071d720fa26d80365d3cccc883726b9a4e00f:8'}]}

    common_field = "liquidations_count"
    protocol_field = ['liquidationCallHistory', 'id']
    transformations = {'liquidations_count': 'array_length'}
    query_elements = {'user_id': ['id'], 'active_loans': [
        'borrowedReservesCount'], 'liquidations_count': ['liquidationCallHistory', 'id']}

    transformer = Transformer()

    liquidations = transformer.transform(
        element,
        common_field,
        protocol_field,
        transformations,
        query_elements
    )

    assert(liquidations == 1)


def test_tx_id_colon():
    element = {'collateralAmount': '8390418', 'collateralReserve': {'decimals': 8, 'symbol': 'WBTC'},
               'id': '0x14e3d6aee390617fde0fb893bcc25401bb981d1321242f3010f26da88b16a099:8',
               'liquidator': '0x645e93859ec63abe0c7fe74f17c07c236ee58799', 'principalAmount':
               '1861472948139816454', 'principalReserve': {'decimals': 18, 'symbol': 'WETH'},
               'timestamp': 1618982463, 'user': {'id': '0xcfd873f19a86b84cfc4916e8623f2486dc83d792'}}

    common_field = "tx_id"
    protocol_field = ['id']
    transformations = {'amount_principal': 'principal_decimals',
                       'amount_collateral': 'collateral_decimals', 'tx_id': 'tx_id_colon'}
    query_elements = {'principal_decimals': ['principalReserve', 'decimals'],
                      'collateral_decimals': ['collateralReserve', 'decimals'], 'tx_id': ['id'],
                      'user': ['user', 'id'], 'token_principal': ['principalReserve', 'symbol'],
                      'token_collateral': ['collateralReserve', 'symbol'], 'amount_principal': ['principalAmount'],
                      'amount_collateral': ['collateralAmount'], 'liquidator': ['liquidator'], 'timestamp': ['timestamp']}

    transformer = Transformer()

    tx = transformer.transform(
        element,
        common_field,
        protocol_field,
        transformations,
        query_elements
    )

    assert(tx == "0x14e3d6aee390617fde0fb893bcc25401bb981d1321242f3010f26da88b16a099")


def test_tx_id_hyphen():
    element = {'id': '0xd522f7c5f29bb6a38d7bbeb08938afb619c8e8a7318a0af0815d4a9d4a8330ac-94',
               'borrower': '0x3f3e305c4ad49271ebda489dd43d2c8f027d2d41',
               'underlyingSymbol': 'USDC', 'amount': '500000', 'blockTime': 1618956398}
    common_field = "tx_id"
    protocol_field = ['id']
    transformations = {'tx_id': 'tx_id_hyphen'}
    query_elements = {'tx_id': ['id'], 'user': ['borrower'],
                      'token': ['underlyingSymbol'], 'amount': ['amount'],
                      'timestamp': ['blockTime']}

    transformer = Transformer()

    tx_id = transformer.transform(
        element,
        common_field,
        protocol_field,
        transformations,
        query_elements
    )

    assert(tx_id == '0xd522f7c5f29bb6a38d7bbeb08938afb619c8e8a7318a0af0815d4a9d4a8330ac')


def test_chainlink_prices():
    element = {'assetPair': {'id': 'ETH/USD'},
               'price': '39178992539', 'timestamp': '1596755816'}
    common_field = "price"
    protocol_field = ['price']
    transformations = {'price': 'chainlink_prices'}
    query_elements = {'pair': ['assetPair', 'id'],
                      'price': ['price'], 'timestamp': ['timestamp']}

    transformer = Transformer()

    price = transformer.transform(
        element,
        common_field,
        protocol_field,
        transformations,
        query_elements
    )

    assert(price == 39178992539 / 10 ** 8)

    element = {'assetPair': {'id': 'ENJ/ETH'},
               'price': '513200000000000', 'timestamp': '1596791153'}
    common_field = "price"
    protocol_field = ['price']
    transformations = {'price': 'chainlink_prices'}
    query_elements = {'pair': ['assetPair', 'id'],
                      'price': ['price'], 'timestamp': ['timestamp']}

    transformer = Transformer()

    price = transformer.transform(
        element,
        common_field,
        protocol_field,
        transformations,
        query_elements
    )

    assert(price == 513200000000000 / 10 ** 18)


def test_remove_token_prefix():
    element = {'id': '0xe5df2dd6aaca27128fdbba8e22c16198e80d21fe11e7d5309077d313d8c2b9c5-238',
               'to': '0x3c71cf9b6335cf604767d3a07fc19664ce7a9052', 'cTokenSymbol': 'cETH',
               'underlyingAmount': '35', 'blockTime': 1618956266}
    common_field = "token"
    protocol_field = ['cTokenSymbol']
    transformations = {'token': 'remove_token_prefix', 'tx_id': 'tx_id_hyphen'}
    query_elements = {'tx_id': ['id'], 'user': ['to'], 'token': ['cTokenSymbol'],
                      'amount': ['underlyingAmount'], 'timestamp': ['blockTime']}

    transformer = Transformer()

    token = transformer.transform(
        element,
        common_field,
        protocol_field,
        transformations,
        query_elements
    )

    assert(token == "ETH")


def test_concat_symbols():
    element = {'amount0In': '0.1', 'amount0Out': '0', 'amount1In': '223212862.542378007',
               'amount1Out': '777931322218.496933182', 'token0': {'symbol': 'WETH'}, 'token1': {'symbol': 'HOKK'},
               'sender': '0x7a250d5630b4cf539739df2c5dacb4c659f2488d',
               'timestamp': '1620079287', 'transaction': {'id': '0xf7bec9d32958801feb1fc06a7dee136e1de844ddbf801ebbb7967b0e39392333'}}

    common_field = "pool_tokens"
    protocol_field = [
        [
            "token0",
            "symbol"
        ],
        [
            "token1",
            "symbol"
        ]
    ]
    transformations = {
        "pool_tokens": "concat_symbols"
    }
    query_elements = {"id": [
        "id"
    ],
        "pool_tokens": [
        [
            "token0",
            "symbol"
        ],
        [
            "token1",
            "symbol"
        ]
    ]}

    transformer = Transformer()

    pool = transformer.transform(
        element,
        common_field,
        protocol_field,
        transformations,
        query_elements
    )

    assert(pool == 'WETH/HOKK')


def test_concat_symbols():
    element = {
        "id": "0x00e5a66d31768318a7cae45dd7a0ef8288d7288d",
        "tokens": [
            {
              "symbol": "WBTC"
            },
            {
                "symbol": "DAI"
            },
            {
                "symbol": "USDC"
            },
            {
                "symbol": "WETH"
            },
            {
                "symbol": "renBTC"
            }
        ]
    }

    common_field = "pool_tokens"
    protocol_field = [
        "tokens",
        "symbol"
    ]
    transformations = {
        "pool_tokens": "concat_list_symbols"
    }
    query_elements = {"id": [
        "id"
    ],
        "pool_tokens": [
        "tokens",
        "symbol"
    ]}

    transformer = Transformer()

    pool = transformer.transform(
        element,
        common_field,
        protocol_field,
        transformations,
        query_elements
    )

    assert(pool == 'WBTC/DAI/USDC/WETH/renBTC')



from deficrawler.dex import Dex


def test_protocol_entities():
    uniswap = Dex(protocol="Uniswap", chain="Ethereum", version=2)
    balancer = Dex(protocol="Balancer", chain="Ethereum", version=1)

    assert(uniswap.supported_entities() == [
           'swap', 'pool'])
    assert(balancer.supported_entities() == [
        'swap', 'pool'])
